#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
'''
This is the code for
    - `Grove - Button <https://www.seeedstudio.com/s/Grove-Button-p-766.html>`_

Examples:

    .. code-block:: python

        from grove.grove_button import GroveButton
        import time

        # connect to pin 5 (slot D5)
        PIN = 5
        button = GroveButton(PIN)

        def on_press(t):
            print('Button is pressed')
        def on_release(t):
            print("Button is released, pressed for {0} seconds".format(round(t,6)))

        button.on_press = on_press
        button.on_release = on_release

        while True:
            time.sleep(1)
'''
import time
from grove.button import Button
from grove.factory import Factory

__all__ = ['GroveButton']

class GroveButton(object):
    '''
    Grove Button class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin):
        # High = pressed
        self.__btn = Factory.getButton("GPIO-HIGH", pin)
        self.__last_time = time.time()
        self.__on_press = None
        self.__on_release = None
        self.__btn.on_event(self, GroveButton.__handle_event)

    @property
    def on_press(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a button pressing.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_press = callback

            get

            .. code-block:: python

                callobj = obj.on_press
        '''
        return self.__on_press

    @on_press.setter
    def on_press(self, callback):
        if not callable(callback):
            return
        self.__on_press = callback

    @property
    def on_release(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a button releasing.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_release = callback

            get

            .. code-block:: python

                callobj = obj.on_release
        '''
        return self.__on_release

    @on_release.setter
    def on_release(self, callback):
        if not callable(callback):
            return
        self.__on_release = callback

    def __handle_event(self, evt):
        dt, self.__last_time = evt["time"] - self.__last_time, evt["time"]
        # print("event index:{} event:{} pressed:{}"
        #      .format(evt["index"], evt["code"], evt["pressed"]))
        if evt["code"] == Button.EV_LEVEL_CHANGED:
            if evt["pressed"]:
                if callable(self.__on_press):
                    self.__on_press(dt)
            else:
                if callable(self.__on_release):
                    self.__on_release(dt)


Grove = GroveButton

def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    button = GroveButton(pin)

    def on_press(t):
        print('Button is pressed')
    def on_release(t):
        print("Button is released, pressed for {0} seconds".format(round(t,6)))

    button.on_press = on_press
    button.on_release = on_release

    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the code for Grove - 6-Axis Accelerometer&Gyroscope (BMI088).
# (https://www.seeedstudio.com/Grove-6-Axis-Accelerometer-Gyroscope-BMI08-p-3188.html)
# which is a 6 DoF(degrees of freedom) High-performance Inertial Measurement Unit(IMU),
# based on BOSCH BMI088.
#
# @author Peter Yang <turmary@126.com>
#
# Grove.py is the library for Grove Base Hat which used to
# connect grove sensors for raspberry pi.
#
'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import division
from __future__ import print_function
from ctypes import *
from ctypes import util
import sys
from grove.i2c import Bus

BMI088_ACCEL_I2C_ADDR = 0x19
BMI088_GYRO_I2C_ADDR  = 0x69

BMI08X_ACCEL_PM_ACTIVE, \
BMI08X_ACCEL_PM_SUSPEND = 0,3

BMI088_ACCEL_RANGE_3G,  \
BMI088_ACCEL_RANGE_6G,  \
BMI088_ACCEL_RANGE_12G, \
BMI088_ACCEL_RANGE_24G  = 0,1,2,3

BMI08X_ACCEL_BW_OSR4, \
BMI08X_ACCEL_BW_OSR2, \
BMI08X_ACCEL_BW_NORMAL= 0,1,2

BMI08X_ACCEL_ODR_12_5_HZ, \
BMI08X_ACCEL_ODR_25_HZ,   \
BMI08X_ACCEL_ODR_50_HZ,   \
BMI08X_ACCEL_ODR_100_HZ,  \
BMI08X_ACCEL_ODR_200_HZ,  \
BMI08X_ACCEL_ODR_400_HZ,  \
BMI08X_ACCEL_ODR_800_HZ,  \
BMI08X_ACCEL_ODR_1600_HZ  = 5, 6, 7, 8, 9, 10, 11, 12

BMI08X_GYRO_PM_NORMAL,      \
BMI08X_GYRO_PM_DEEP_SUSPEND,\
BMI08X_GYRO_PM_SUSPEND    = 0x0, 0x20, 0x80

BMI08X_GYRO_RANGE_2000_DPS, \
BMI08X_GYRO_RANGE_1000_DPS, \
BMI08X_GYRO_RANGE_500_DPS,  \
BMI08X_GYRO_RANGE_250_DPS,  \
BMI08X_GYRO_RANGE_125_DPS   = 0, 1, 2, 3, 4

BMI08X_GYRO_BW_532_ODR_2000_HZ, \
BMI08X_GYRO_BW_230_ODR_2000_HZ, \
BMI08X_GYRO_BW_116_ODR_1000_HZ, \
BMI08X_GYRO_BW_47_ODR_400_HZ,   \
BMI08X_GYRO_BW_23_ODR_200_HZ,   \
BMI08X_GYRO_BW_12_ODR_100_HZ,   \
BMI08X_GYRO_BW_64_ODR_200_HZ,   \
BMI08X_GYRO_BW_32_ODR_100_HZ,   \
BMI08X_GYRO_ODR_RESET_VAL   =   0, 1, 2, 3, 4, 5, 6, 7, 0x80


__c_module = "bmi088"

try:
    _ = util.find_library(__c_module)
    _bmi = cdll.LoadLibrary(_)
except Exception:
    print("Error: module lib{}.so unusable, please install lib{}".
          format(__c_module, __c_module))
    sys.exit(1)

class BMI08xCfg(Structure):
    _fields_ = [("power", c_ubyte), \
                ("range", c_ubyte), \
                ("bw",    c_ubyte), \
                ("odr",   c_ubyte)]



class GroveAccelGyroBMI088(object):
    def __init__(self, acc_addr = BMI088_ACCEL_I2C_ADDR, gyro_addr = BMI088_GYRO_I2C_ADDR):
        _bmi.rpi_bmi088_alloc.restype = c_char_p
        self._dev = _bmi.rpi_bmi088_alloc()
        dev_path = b"/dev/i2c-%d" %(Bus().bus)
        

        accel_cfg = BMI08xCfg(BMI08X_ACCEL_PM_ACTIVE,
                              BMI088_ACCEL_RANGE_6G,
                              BMI08X_ACCEL_BW_NORMAL,
                              BMI08X_ACCEL_ODR_100_HZ)
        gyro_cfg = BMI08xCfg(BMI08X_GYRO_PM_NORMAL,
                              BMI08X_GYRO_RANGE_1000_DPS,
                              BMI08X_GYRO_BW_23_ODR_200_HZ,
                              BMI08X_GYRO_BW_23_ODR_200_HZ)
        _bmi.rpi_bmi088_init.argtypes = [c_char_p,c_char_p,c_int,c_int,POINTER(BMI08xCfg),POINTER(BMI08xCfg)]
        _bmi.rpi_bmi088_init(self._dev,
                             dev_path,
                             acc_addr,
                             gyro_addr,
                             byref(accel_cfg),
                             byref(gyro_cfg))

    def __del__(self):
        _bmi.rpi_bmi088_free(self._dev)

    def get_sensor_time(self):
        return _bmi.rpi_bmi088_get_sensor_time(self._dev)

    def get_accel(self):
        x, y, z = c_double(), c_double(), c_double()
        _bmi.rpi_bmi088_get_accel(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

    def get_gyro(self):
        x, y, z = c_double(), c_double(), c_double()
        _bmi.rpi_bmi088_get_gyro(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

Grove = GroveAccelGyroBMI088

def main():
    import time
    print(\
""" Make sure 6-Axis-Accelerometer-Gyroscope(BMI088)
   inserted in one I2C slot of Grove-Base-Hat
""")

    snr = GroveAccelGyroBMI088()
    while True:
        tm = snr.get_sensor_time()
        # don't ask me why 26000, it's magic.
        print("Sensor time: {:.2f} S".format(tm / 26000.0))
        x, y, z = snr.get_accel()
        print(" AX = %7.2f mg  AY = %7.2f mg  AZ = %7.2f mg" % (x, y, z))
        x, y, z = snr.get_gyro()
        print(" GX = %7.2f dps GY = %7.2f dps GZ = %7.2f dps" % (x, y, z))
        time.sleep(1.0)

if __name__ == '__main__':
    main()



"""
License
The MIT License (MIT)

Copyright (C) 2020 Ville Laine, Aikuiskoulutus Taitaja

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

# Necessary functions
def calc_crc8(bytes_list):
    # CRC init value
    crc = 0xff

    # loop through bytes
    for i in bytes_list:
        for bit in range(0, 8):
            if (i ^ crc) & 0x80:
                crc = (crc << 1) ^ 0x31
            else:
                crc = (crc << 1)
            i = i << 1
        crc = crc & 0xFF

    return crc


def validate_crc8(int_list):
    ok = 1
    pos = 0
    values = []
    size = 3

    # Split list
    while len(int_list) > size:
        pice = int_list[:size]
        values.append(pice)
        int_list = int_list[size:]
        values.append(int_list)

    # loop through all words and respective crc [HByte, LByte, CRC]
    for value in values:
        pos = pos + 1

        # CRC init value
        crc = 0xff

        # loop through only HByte and LByte
        for i in value[0:2]:
            for bit in range(0, 8):
                if (i ^ crc) & 0x80:
                    crc = (crc << 1) ^ 0x31
                else:
                    crc = (crc << 1)
                i = i << 1
            crc = crc & 0xFF

        # Check CRC status when word processed and exit if crc checksum fails
        if crc != value[2]:
            ok = 0
            # crc_status = [ok, pos]
            print("crc check failed!")
            return [ok, pos]

    if ok == 1:
        pos = 0
    # crc_status = [ok, pos]
    return [ok, pos]


# Convert two bytes to one int(word) = int(HByte+LByte)
# example (HByte = 0x0a, LByte = 0xf4) = 0x0af4 = 2804
# Returns hexadecimal representation
def bytes_to_int(bytes_list):
    return_str = ""

    for i in bytes_list:
        temp_str = str(hex(i))[2:]

        # Add trailing zero if value is 0x0
        if int(temp_str, 16) == 0:
            temp_str = temp_str + "0"
        return_str = return_str + temp_str

    return int(return_str, 16)


# Convert int(word) = int(HByte+LByte) into two bytes
# example 2804 = 0x0af4 = (HByte = 0x0a, LByte = 0xf4)
def int_to_bytes(number):
    tmp_str = str(hex(number))
    hbyte = "0x"+tmp_str[2:4]
    lbyte = "0x"+tmp_str[-2:]
    return [int(hbyte, 16), int(lbyte, 16)]




try:
    from grove.gpio.gpio_rpi import GPIO
except ImportError:
    print('RPi.GPIO is not available.Exit.')
    exit(1)

from grove.gpio.wrapper import GPIOWrapper

__all__ = ['GPIO', 'GPIOWrapper']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
'''
This is the grove.gpio.GPIO implemented by RPi.GPIO.
'''
import RPi.GPIO

__all__ = ['GPIO']

RPi.GPIO.setwarnings(False)
RPi.GPIO.setmode(RPi.GPIO.BCM)


class GPIO(object):
    OUT = RPi.GPIO.OUT
    IN = RPi.GPIO.IN
    
    def __init__(self, pin, direction=None):
        self.pin = pin
        if direction is not None:
            self.dir(direction)

        self._event_handle = None

    def dir(self, direction):
        RPi.GPIO.setup(self.pin, direction)

    def write(self, output):
        RPi.GPIO.output(self.pin, output)

    def read(self):
        return RPi.GPIO.input(self.pin)

    def _on_event(self, pin):
        value = self.read()
        if self._event_handle:
            self._event_handle(pin, value)

    @property
    def on_event(self):
        return self._event_handle

    @on_event.setter
    def on_event(self, handle):
        if not callable(handle):
            return

        if self._event_handle is None:
            RPi.GPIO.add_event_detect(self.pin, RPi.GPIO.BOTH, self._on_event)

        self._event_handle = handle



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd. 
#
'''
'''

__all__ = ["Button"]

class Button(object):
    '''
    Button Base Class

    provide event callback ability to derived class

    Args:
        pin(int): optional, for GPIO type button, it's the GPIO pin number.
    '''
    # event bits
    EV_RAW_STATUS    = 1 << 0
    EV_SINGLE_CLICK  = 1 << 1
    EV_DOUBLE_CLICK  = 1 << 2
    EV_LONG_PRESS    = 1 << 3
    EV_LEVEL_CHANGED = 1 << 4
    # EV_HAS           = 1 << 31

    pins = []

    def __init__(self, pin = 0):
        self.__on_obj = None
        self.__on_event = None
        self.__event = 0
        self.pins.append(pin)
        # To use with button array
        self._index = self.pins.index(pin)

    def get_on_event(self):
        '''
        Get the event receiving object and callback member function
        
        Returns:
            (obj, event_callback): a pair consist of event receiving object
                                   and callback member function
        '''
        return self.__on_obj, self.__on_event

    def on_event(self, obj, callback):
        '''
        Set the event receiving object and it'callback member function

        Args:
            obj(object)       : the object to receiving event
            callback(callable): a member function of `obj`,
                will be called when there is button event

                callback prototype:
                    callback(obj, evt_dict)

                callback argument:
                    Args:
                        obj(object)   : the object that is `obj` argument of :class:`on_event`

                        evt_dict(dict): the event dictionary include items:
                                        - index
                                        - code
                                        - pressed
                                        - time

                    Returns: none
        '''
        if not obj:
            return
        if not callable(callback):
            return
        self.__on_obj, self.__on_event = obj, callback

    def is_pressed(self, index = 0):
        '''
        Get the button status if it's being pressed ?

        Args:
            index(int): optional, the index number of which button to be checked.
                        must be specified only if it's a multiple button device.

        Returns:
            (bool): True  if the button is being pressed.
                    False if not.
        '''
        return False

    # call by derivate class
    def _send_event(self, event, pressed, tm):
        if not callable(self.__on_event):
            return

        evt = {
                'index': self._index,
                'code' : event,
                'pressed': pressed,
                'time' : tm,
        }
        self.__on_event(self.__on_obj, evt)



#!/usr/bin/env python
#
# Grove Library for I2C-Motor-Driver with Stepper Motor

from __future__ import division
from grove.i2c import Bus

__all__ = ["StepperMotor", "I2CStepperMotor"]

class StepperMotor(object):
    DC_SPEED_MAX         = 100
    _DIR_ANTI_CLKWISE    = 0
    _DIR_CLKWISE         = 1

    def __init__(self, arguments):
        # default Dirction
        self._dir = self._DIR_CLKWISE
        # Speed Varation Ratio, 1.0 is unapplicable
        # Most time it's reductor.
        self._var_ratio = arguments["var-ratio"]
        # Stride Angle
        self._stride_angle = arguments["stride-angle"]
        self._rpm_max      = arguments.get("rpm-max", 1.0)
        self._name         = arguments.get("name", "StepperMotor")

    @property
    def speed_max(self):
        return self._rpm_max

    @property
    def name(self):
        return self._name

    # To be derived
    # def _rotate(self, angle = None):
    #    return None

    def rotate(self, angle = None):
        "specify the stepper motor rotating angle,"
        " query the angle left if angle missed"
        "    angle  --- 1 cycle = 360 degrees, positive float or int acceptable"
        "    return angle left"
        return self._rotate(angle)

    # To be derived
    def _speed(self, rpm):
        pass

    def speed(self, rpm = None):
        "set or get stepper motor speed & direction"
        "    rpm    --- revolutions per minute, float or int acceptable"
        "               > 0 clockwise, < 0 anti-clockwise"
        "    return rpm value stored"
        if not rpm is None:
            self._rpm = rpm
            self._speed(rpm)
        return self._rpm

    # To be derived
    def _enable(self, en):
        pass

    def enable(self, en = None):
        "enable or disable the stepper motor"
        "    en    --- True for enable, False for disable"
        "    return disable or enable state stored"
        if not en is None:
            self._en = en
            self._enable(en)
        return self._en

    def _steps2angle(self, steps):
        angle = steps * self._stride_angle / self._var_ratio
        return angle

    def _angle2steps(self, angle):
        steps = angle * self._var_ratio / self._stride_angle
        return int(steps)



import time

class I2CStepperMotor(StepperMotor):
    __REG_GET_PID       = 0x00
    __REG_GET_VID       = 0x01
    __REG_GET_VER       = 0x02
    __REG_STP_EN        = 0x1A
    __REG_STP_DIS       = 0x1B
    __REG_STP_RUN       = 0x1C
    __REG_STP_INTERVAL  = 0x1D
    __REG_SEQ_LEN       = 0x20
    __REG_SEQ_XET       = 0x21
    __REG_SET_SPEED     = 0x82
    __REG_SET_FREQ      = 0x84
    __REG_SET_A         = 0xA1
    __REG_SET_B         = 0xA5
    __REG_SET_DIR       = 0xAA

    def __init__(self, arguments, address = 0x0F):
        super(I2CStepperMotor, self).__init__(arguments)
        self._addr = address
        self._bus = Bus()
        self._ang_left = 0
        self._load_seq(arguments["sequences"])

    def __del__(self):
        self.set_speed(0, 0)
        pass

    #Maps speed from 0-100 to 0-255
    def _map_vals(self,value, leftMin, leftMax, rightMin, rightMax):
        #http://stackoverflow.com/questions/1969240/mapping-a-range-of-values-to-another
        # Figure out how 'wide' each range is
        leftSpan = leftMax - leftMin
        rightSpan = rightMax - rightMin

        # Convert the left range into a 0-1 range (float)
        valueScaled = float(value - leftMin) / float(leftSpan)

        # Convert the 0-1 range into a value in the right range.
        return int(rightMin + (valueScaled * rightSpan))
        
    #Set motor speed
    def set_speed(self, speed1 = 0, speed2 = 0):
        s1 = self._map_vals(speed1, 0, 100, 0, 255)
        s2 = self._map_vals(speed2, 0, 100, 0, 255)
        self._bus.write_i2c_block_data(self._addr, self.__REG_SET_SPEED, [s1, s2])
        time.sleep(.02)
    
    #Set motor direction
    def set_dir(self, clock_wise1 = True, clock_wise2 = True):
        dir1 = 0b10 if clock_wise1 else 0b01
        dir2 = 0b10 if clock_wise2 else 0b01
        dir = (dir2 << 2) | dir1
        self._bus.write_i2c_block_data(self._addr, self.__REG_SET_DIR, [dir, 0])
        time.sleep(.02)

    def _load_seq(self, seq):
        length = len(seq)
        self._bus.write_word_data(self._addr, self.__REG_SEQ_LEN, length)
        for i in range(len(seq)):
            self._bus.write_word_data(self._addr, self.__REG_SEQ_XET, seq[i])

    def _enable(self, en):
        cmd = self.__REG_STP_EN if en else self.__REG_STP_DIS
        if en:
            self.set_speed(self.DC_SPEED_MAX, self.DC_SPEED_MAX)
        else:
            self.set_speed(0, 0)
        self._bus.write_i2c_block_data(self._addr, cmd, [self._dir, 0])
        time.sleep(0.001)

    def _speed(self, rpm):
        self._dir = self._DIR_CLKWISE if rpm >= 0 else self._DIR_ANTI_CLKWISE
        # absolute angle per second
        aps = abs(rpm) * 360.0 / 60.0
        # steps per second, include reductor ratio.
        sps = self._angle2steps(aps)
        period = int(1000000 / sps) # us
        period = period // 10       # STP_INTERVAL, 10 us
        # print("period = %d us" % (period * 10))
        self._bus.write_word_data(self._addr, self.__REG_STP_INTERVAL, period)
        time.sleep(0.001)

    def _rotate(self, angle = None):
        if not angle is None:
            angle = abs(angle)
            self._ang_left = angle
            steps = self._angle2steps(angle)
            # print("steps set = {}".format(steps))
            self._bus.write_word_data(self._addr, self.__REG_STP_RUN, steps)
            time.sleep(0.001)
            return angle
        while True:
            # reading interface unstable when working
            try:
                steps = self._bus.read_word_data(self._addr, self.__REG_STP_RUN)
                # print("steps left = {}".format(steps))
                ang_left = self._steps2angle(steps)
                if ang_left > self._ang_left:
                    time.sleep(0.01)
                    continue
                self._ang_left = ang_left
                return ang_left
            except IOError:
                continue


def main():
    print("Make sure I2C-Motor-Driver inserted")
    print("  in one I2C slot of Grove-Base-Hat")

    arguments_28BYJ48 = {
        'var-ratio'   : 64,
        'stride-angle': 5.625,
        'rpm-max'     : 12,
        'sequences'   :
            # clockwise
            [ 0b0001, 0b0011, 0b0010, 0b0110, 0b0100, 0b1100, 0b1000, 0b1001 ]
            # [ 0b1110, 0b1101, 0b1011, 0b0111 ]
            # anti-clockwise
            # [ 0b1001, 0b1000, 0b1100, 0b0100, 0b0110, 0b0010, 0b0011, 0b0001 ]
    }

    # motor = I2CStepperMotor(arguments_YH42BYGH40)
    motor = I2CStepperMotor(arguments_28BYJ48)
    ANGLE = 360 # rotate 360 degrees = 1 cycle
    motor.rotate(ANGLE)
    SPEED = 12 # speed set to 12 rpm, clockwise
    # anti-clockwise use negative value (-12)
    motor.speed(SPEED)
    # enable the motor, begin to run
    motor.enable(True)

    seconds = ANGLE / 360 / SPEED * 60
    print("Motor rotate {} degrees, time = {:.2f}".format(ANGLE, time.time()))
    print("      with speed {} RPM".format(SPEED))
    print("      will be stop after {:4.2f} seconds".format(seconds))

    while True:
        left = motor.rotate()
        if left < 1e-5:
            break
        print("Angle left {:6.2f} degrees, time = {:.2f}".format(left, time.time()))
        time.sleep(1.0)

    print("Motor run ended, time = {:.2f} !".format(time.time()))

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Temperature Sensor <https://www.seeedstudio.com/Grove-Temperature-Sensor-p-774.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.factory import Factory

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = Factory.getTemper("NTC-ADC", PIN)

        print('Detecting temperature...')
        while True:
            print('{} Celsius'.format(sensor.temperature))
            time.sleep(1)
'''
import sys
import time
from grove.factory import Factory


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = Factory.getTemper("NTC-ADC", pin)

    print('Detecting temperature...')
    while True:
        print('{} Celsius'.format(sensor.temperature))
        time.sleep(1)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Rotary Angle Sensor(P) <https://www.seeedstudio.com/Grove-Rotary-Angle-Sensor(P)-p-1242.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_rotary_angle_sensor import GroveRotaryAngleSensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveRotaryAngleSensor(PIN)

        while True:
            print('Rotary Value: {}'.format(sensor.value))
            time.sleep(.2)

'''
import math
import time
from grove.adc import ADC

__all__ = ["GroveRotaryAngleSensor"]

class GroveRotaryAngleSensor(ADC):
    '''
    Grove Rotary Angle Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()
    
    @property
    def value(self):
        '''
        Get the rotary angle value, max angle is 100.0%

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channel)


Grove = GroveRotaryAngleSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveRotaryAngleSensor(pin)

    while True:
        print('Rotary Value: {}'.format(sensor.value))
        time.sleep(.2)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
Display Base Class
'''

# sphinx autoapi required
__all__ = [
    "Display",
    "TYPE_CHAR",
    "TYPE_GRAY",
    "TYPE_COLOR",
    "MAX_GRAY"
]

TYPE_CHAR  = 0
TYPE_GRAY  = 1
TYPE_COLOR = 2

MAX_GRAY = 100

class Display(object):
    '''
    All display devices should inherit this virtual class,
    which provide infrastructure such as cursor and backlight inteface, etc.
    '''
    def __init__(self):
        self._cursor = False
        self._backlight = False

    # To be derived
    def _cursor_on(self, en):
        pass

    def cursor(self, enable = None):
        '''
        Enable or disable the backlight on display device,
        not all device support it.

        Args:
            enable (bool): Optional, ``True`` to enable, ``Flase`` to disable.
                           if not provided, only to get cursor status.

        Returns:
            bool: cursor status, ``True`` - on, ``False`` - off.
        '''
        if type(enable) == bool:
            self._cursor = enable
            self._cursor_on(enable)
        return self._cursor

    # To be derived
    def _backlight_on(self, en):
        pass

    def backlight(self, enable = None):
        '''
        Enable or disable the cursor on display device,
        not all device support it.

        Args:
            enable (bool): Optional, ``True`` to enable, ``Flase`` to disable.
                           if not provided, only to get cursor status.

        Returns:
            bool: backlight status, ``True`` - on, ``False`` - off.
        '''
        if type(enable) == bool:
            self._backlight = enable
            self._backlight_on(enable)
        return self._backlight




#!/usr/bin/env python
#
# This is the code for Grove - 1-Wire Thermocouple Amplifier (MAX31850K).
# (https://www.seeedstudio.com/Grove-1-Wire-Thermocouple-Amplifier-MAX31850-p-3159.html)
# which is a thermocouple-to-digital converters with
# 14-bit resolution and cold-junction compensation.
#
# author: Peter Yang <turmary@126.com>
#
# Grove.py is the library for Grove Base Hat which used to
# connect grove sensors for raspberry pi.
#
#
'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import print_function
from __future__ import division
import time
import glob
import sys
import os
import re

class Grove1WireThermoAmpMAX31850(object):
    def __init__(self,
        path = "/sys/devices/w1_bus_master1/*/w1_slave"
    ):
        self._devs = glob.glob(path)

    def temperature(self):
        if not len(self._devs):
            return None
        for line in os.popen("cat " + self._devs[0]):
            # lines likes
            #78 01 b0 17 f0 ff ff ff 9f : crc=9f YES
            #78 01 b0 17 f0 ff ff ff 9f t=23500
            match = re.match("^(?:[0-9a-f][0-9a-f] ){9}t=([0-9]+)", line)
            if match is None: continue
            r = int(match.group(1))
            # print("result = %d" % r)
            return r / 1000.0
        return None


    get_temperature = temperature
    read = temperature


Grove = Grove1WireThermoAmpMAX31850

# configuration
pin = 5

def main():
    print(\
""" Make sure Grove-1-Wire-Thermocouple-Amplifier(MAX31850K)
   inserted in *** slot D{} ***
""".format(pin))

    from grove import helper
    helper.root_check()

    print("This program need overlay w1-gpio")
    from grove.helper import OverlayHelper
    w1_path = "/sys/devices/w1_bus_master1"
    oh = OverlayHelper(w1_path,
                       "w1-gpio",
                       "gpiopin=%d" % pin)
    print(oh)
    if not oh.is_installed():
        print("install overlay {} ...".format(oh.name))
        oh.install()

    print("Search MAX31850 devices ...")
    import os
    os.system("echo 1 > " + w1_path + "/w1_master_search")

    snr = Grove1WireThermoAmpMAX31850(w1_path + "/*/w1_slave")
    while True:
        print("Temperature: {:.2f} C".format(snr.read()))
        time.sleep(1.0)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
'''
This is the code for
    - `Grove - Switch(P) <https://www.seeedstudio.com/Grove-Switch-p-1252.html>`_

Examples:

    .. code-block:: python

        from grove.grove_switch import GroveSwitch
        import time

        # connect to pin 5 (slot D5)
        PIN = 5
        swicth = GroveSwitch(PIN)

        while True:
            if swicth.state:
                print("high")
            else:
                print("low")
            time.sleep(1)

'''
import time
from grove.gpio import GPIO

__all__ = ['GroveSwitch']

class GroveSwitch(GPIO):
    '''
    Grove Switch class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin):
        super(GroveSwitch, self).__init__(pin, GPIO.IN)

    @property
    def state(self):
        '''
        Read only property to get switch status.

        Returns:
            (bool):

                - True:  switch high
                - False: switch low

        '''
        return bool(super(GroveSwitch, self).read())


Grove = GroveSwitch


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    switch = GroveSwitch(pin)

    while True:
        if switch.state:
            print("HIGH")
        else:
            print("LOW")
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# GrovePi Library for using the Grove - Gesture Sensor v1.0(http://www.seeedstudio.com/depot/Grove-Gesture-p-2463.html)
#
# The GrovePi connects the Raspberry Pi and Grove sensors.  You can learn more about GrovePi here:  http://www.dexterindustries.com/GrovePi
#
# Have a question about this library?  Ask on the forums here:  http://forum.dexterindustries.com/c/grovepi
#
# History
# ------------------------------------------------
# Author	Date      		Comments
# Karan		31 Dec 15	  	Initial Authoring
# Lambor    11 July 18      Use smbus2
#
# Code derived from the basic Arduino library for the Gesture Sensor by Seeed: https://github.com/Seeed-Studio/Gesture_PAJ7620
'''
## License

The MIT License (MIT)

GrovePi for the Raspberry Pi: an open source platform for connecting Grove Sensors to the Raspberry Pi.
Copyright (C) 2017  Dexter Industries

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import time,sys
import RPi.GPIO as GPIO
from grove.i2c import Bus

# use the bus that matches your raspi version
bus = Bus()


class gesture:
	#Registers and variables for the gesture sensor
	GES_REACTION_TIME		=.500				# You can adjust the reaction time according to the actual circumstance.
	GES_ENTRY_TIME			=.800				# When you want to recognize the Forward/Backward gestures, your gestures' reaction time must less than GES_ENTRY_TIME(0.8s). 
	GES_QUIT_TIME			=1.000
	
	BANK0 = 0
	BANK1 = 1
	
	PAJ7620_ADDR_BASE =0x00

	#REGISTER BANK SELECT
	PAJ7620_REGITER_BANK_SEL		=(PAJ7620_ADDR_BASE + 0xEF)	#W

	#DEVICE ID
	PAJ7620_ID  =0x73

	#REGISTER BANK 0
	PAJ7620_ADDR_SUSPEND_CMD		=(PAJ7620_ADDR_BASE + 0x3)	#W
	PAJ7620_ADDR_GES_PS_DET_MASK_0	=(PAJ7620_ADDR_BASE + 0x41)	#RW
	PAJ7620_ADDR_GES_PS_DET_MASK_1	=(PAJ7620_ADDR_BASE + 0x42)	#RW
	PAJ7620_ADDR_GES_PS_DET_FLAG_0	=(PAJ7620_ADDR_BASE + 0x43)	#R
	PAJ7620_ADDR_GES_PS_DET_FLAG_1	=(PAJ7620_ADDR_BASE + 0x44)	#R
	PAJ7620_ADDR_STATE_INDICATOR	=(PAJ7620_ADDR_BASE + 0x45)	#R
	PAJ7620_ADDR_PS_HIGH_THRESHOLD	=(PAJ7620_ADDR_BASE + 0x69)	#RW
	PAJ7620_ADDR_PS_LOW_THRESHOLD	=(PAJ7620_ADDR_BASE + 0x6A)	#RW
	PAJ7620_ADDR_PS_APPROACH_STATE	=(PAJ7620_ADDR_BASE + 0x6B)	#R
	PAJ7620_ADDR_PS_RAW_DATA		=(PAJ7620_ADDR_BASE + 0x6C)	#R
                                    
	#REGISTER BANK 1                
	PAJ7620_ADDR_PS_GAIN			=(PAJ7620_ADDR_BASE + 0x44)	#RW
	PAJ7620_ADDR_IDLE_S1_STEP_0		=(PAJ7620_ADDR_BASE + 0x67)	#RW
	PAJ7620_ADDR_IDLE_S1_STEP_1		=(PAJ7620_ADDR_BASE + 0x68)	#RW
	PAJ7620_ADDR_IDLE_S2_STEP_0		=(PAJ7620_ADDR_BASE + 0x69)	#RW
	PAJ7620_ADDR_IDLE_S2_STEP_1		=(PAJ7620_ADDR_BASE + 0x6A)	#RW
	PAJ7620_ADDR_OP_TO_S1_STEP_0	=(PAJ7620_ADDR_BASE + 0x6B)	#RW
	PAJ7620_ADDR_OP_TO_S1_STEP_1	=(PAJ7620_ADDR_BASE + 0x6C)	#RW
	PAJ7620_ADDR_OP_TO_S2_STEP_0	=(PAJ7620_ADDR_BASE + 0x6D)	#RW
	PAJ7620_ADDR_OP_TO_S2_STEP_1	=(PAJ7620_ADDR_BASE + 0x6E)	#RW
	PAJ7620_ADDR_OPERATION_ENABLE	=(PAJ7620_ADDR_BASE + 0x72)	#RW

	#PAJ7620_REGITER_BANK_SEL
	PAJ7620_BANK0=0
	PAJ7620_BANK1=1
	
	#PAJ7620_ADDR_SUSPEND_CMD
	PAJ7620_I2C_WAKEUP	=1
	PAJ7620_I2C_SUSPEND	=0
	
	#PAJ7620_ADDR_OPERATION_ENABLE
	PAJ7620_ENABLE=1
	PAJ7620_DISABLE=0
	
	#ADC, delete
	REG_ADDR_RESULT = 0x00
	REG_ADDR_ALERT  = 0x01
	REG_ADDR_CONFIG = 0x02
	REG_ADDR_LIMITL = 0x03
	REG_ADDR_LIMITH = 0x04
	REG_ADDR_HYST   = 0x05
	REG_ADDR_CONVL  = 0x06
	REG_ADDR_CONVH  = 0x07

	GES_RIGHT_FLAG				=1<<0
	GES_LEFT_FLAG				=1<<1
	GES_UP_FLAG					=1<<2
	GES_DOWN_FLAG				=1<<3
	GES_FORWARD_FLAG			=1<<4
	GES_BACKWARD_FLAG			=1<<5
	GES_CLOCKWISE_FLAG			=1<<6
	GES_COUNT_CLOCKWISE_FLAG	=1<<7
	GES_WAVE_FLAG				=1<<0
	
	#Gesture output
	FORWARD 		= 1
	BACKWARD 		= 2
	RIGHT			= 3
	LEFT			= 4
	UP				= 5
	DOWN			= 6
	CLOCKWISE		= 7
	ANTI_CLOCKWISE	= 8
	WAVE			= 9
	
	#Initial register state
	initRegisterArray=(	[0xEF,0x00],
						[0x32,0x29],
						[0x33,0x01],
						[0x34,0x00],
						[0x35,0x01],
						[0x36,0x00],
						[0x37,0x07],
						[0x38,0x17],
						[0x39,0x06],
						[0x3A,0x12],
						[0x3F,0x00],
						[0x40,0x02],
						[0x41,0xFF],
						[0x42,0x01],
						[0x46,0x2D],
						[0x47,0x0F],
						[0x48,0x3C],
						[0x49,0x00],
						[0x4A,0x1E],
						[0x4B,0x00],
						[0x4C,0x20],
						[0x4D,0x00],
						[0x4E,0x1A],
						[0x4F,0x14],
						[0x50,0x00],
						[0x51,0x10],
						[0x52,0x00],
						[0x5C,0x02],
						[0x5D,0x00],
						[0x5E,0x10],
						[0x5F,0x3F],
						[0x60,0x27],
						[0x61,0x28],
						[0x62,0x00],
						[0x63,0x03],
						[0x64,0xF7],
						[0x65,0x03],
						[0x66,0xD9],
						[0x67,0x03],
						[0x68,0x01],
						[0x69,0xC8],
						[0x6A,0x40],
						[0x6D,0x04],
						[0x6E,0x00],
						[0x6F,0x00],
						[0x70,0x80],
						[0x71,0x00],
						[0x72,0x00],
						[0x73,0x00],
						[0x74,0xF0],
						[0x75,0x00],
						[0x80,0x42],
						[0x81,0x44],
						[0x82,0x04],
						[0x83,0x20],
						[0x84,0x20],
						[0x85,0x00],
						[0x86,0x10],
						[0x87,0x00],
						[0x88,0x05],
						[0x89,0x18],
						[0x8A,0x10],
						[0x8B,0x01],
						[0x8C,0x37],
						[0x8D,0x00],
						[0x8E,0xF0],
						[0x8F,0x81],
						[0x90,0x06],
						[0x91,0x06],
						[0x92,0x1E],
						[0x93,0x0D],
						[0x94,0x0A],
						[0x95,0x0A],
						[0x96,0x0C],
						[0x97,0x05],
						[0x98,0x0A],
						[0x99,0x41],
						[0x9A,0x14],
						[0x9B,0x0A],
						[0x9C,0x3F],
						[0x9D,0x33],
						[0x9E,0xAE],
						[0x9F,0xF9],
						[0xA0,0x48],
						[0xA1,0x13],
						[0xA2,0x10],
						[0xA3,0x08],
						[0xA4,0x30],
						[0xA5,0x19],
						[0xA6,0x10],
						[0xA7,0x08],
						[0xA8,0x24],
						[0xA9,0x04],
						[0xAA,0x1E],
						[0xAB,0x1E],
						[0xCC,0x19],
						[0xCD,0x0B],
						[0xCE,0x13],
						[0xCF,0x64],
						[0xD0,0x21],
						[0xD1,0x0F],
						[0xD2,0x88],
						[0xE0,0x01],
						[0xE1,0x04],
						[0xE2,0x41],
						[0xE3,0xD6],
						[0xE4,0x00],
						[0xE5,0x0C],
						[0xE6,0x0A],
						[0xE7,0x00],
						[0xE8,0x00],
						[0xE9,0x00],
						[0xEE,0x07],
						[0xEF,0x01],
						[0x00,0x1E],
						[0x01,0x1E],
						[0x02,0x0F],
						[0x03,0x10],
						[0x04,0x02],
						[0x05,0x00],
						[0x06,0xB0],
						[0x07,0x04],
						[0x08,0x0D],
						[0x09,0x0E],
						[0x0A,0x9C],
						[0x0B,0x04],
						[0x0C,0x05],
						[0x0D,0x0F],
						[0x0E,0x02],
						[0x0F,0x12],
						[0x10,0x02],
						[0x11,0x02],
						[0x12,0x00],
						[0x13,0x01],
						[0x14,0x05],
						[0x15,0x07],
						[0x16,0x05],
						[0x17,0x07],
						[0x18,0x01],
						[0x19,0x04],
						[0x1A,0x05],
						[0x1B,0x0C],
						[0x1C,0x2A],
						[0x1D,0x01],
						[0x1E,0x00],
						[0x21,0x00],
						[0x22,0x00],
						[0x23,0x00],
						[0x25,0x01],
						[0x26,0x00],
						[0x27,0x39],
						[0x28,0x7F],
						[0x29,0x08],
						[0x30,0x03],
						[0x31,0x00],
						[0x32,0x1A],
						[0x33,0x1A],
						[0x34,0x07],
						[0x35,0x07],
						[0x36,0x01],
						[0x37,0xFF],
						[0x38,0x36],
						[0x39,0x07],
						[0x3A,0x00],
						[0x3E,0xFF],
						[0x3F,0x00],
						[0x40,0x77],
						[0x41,0x40],
						[0x42,0x00],
						[0x43,0x30],
						[0x44,0xA0],
						[0x45,0x5C],
						[0x46,0x00],
						[0x47,0x00],
						[0x48,0x58],
						[0x4A,0x1E],
						[0x4B,0x1E],
						[0x4C,0x00],
						[0x4D,0x00],
						[0x4E,0xA0],
						[0x4F,0x80],
						[0x50,0x00],
						[0x51,0x00],
						[0x52,0x00],
						[0x53,0x00],
						[0x54,0x00],
						[0x57,0x80],
						[0x59,0x10],
						[0x5A,0x08],
						[0x5B,0x94],
						[0x5C,0xE8],
						[0x5D,0x08],
						[0x5E,0x3D],
						[0x5F,0x99],
						[0x60,0x45],
						[0x61,0x40],
						[0x63,0x2D],
						[0x64,0x02],
						[0x65,0x96],
						[0x66,0x00],
						[0x67,0x97],
						[0x68,0x01],
						[0x69,0xCD],
						[0x6A,0x01],
						[0x6B,0xB0],
						[0x6C,0x04],
						[0x6D,0x2C],
						[0x6E,0x01],
						[0x6F,0x32],
						[0x71,0x00],
						[0x72,0x01],
						[0x73,0x35],
						[0x74,0x00],
						[0x75,0x33],
						[0x76,0x31],
						[0x77,0x01],
						[0x7C,0x84],
						[0x7D,0x03],
						[0x7E,0x01])

	#Enable debug message
	debug=0
	
	#Initialize the sensors
	def init(self):
		time.sleep(.001)
		self.paj7620SelectBank(self.BANK0)
		self.paj7620SelectBank(self.BANK0)
		
		data0 = self.paj7620ReadReg(0, 1)[0]
		data1 = self.paj7620ReadReg(1, 1)[0]
		if self.debug:
			print("data0:",data0,"data1:",data1)
		if data0 != 0x20  :#or data1 <> 0x76:
			print("Error with sensor")
			#return 0xff
		if data0 == 0x20:
			print("wake-up finish.")
			
		for i in range(len(self.initRegisterArray)):
			self.paj7620WriteReg(self.initRegisterArray[i][0],self.initRegisterArray[i][1])
		
		self.paj7620SelectBank(self.BANK0)
		
		print("Paj7620 initialize register finished.")
		
	#Write a byte to a register on the Gesture sensor
	def paj7620WriteReg(self,addr,cmd):
		bus.write_word_data(self.PAJ7620_ID, addr, cmd)
		
	#Select a register bank on the Gesture Sensor
	def paj7620SelectBank(self,bank):
		if bank==self.BANK0:
			self.paj7620WriteReg(self.PAJ7620_REGITER_BANK_SEL, self.PAJ7620_BANK0)
			
	#Read a block of bytes of length "qty" starting at address "addr" from the Gesture sensor
	def paj7620ReadReg(self,addr,qty):
		return bus.read_i2c_block_data(self.PAJ7620_ID, addr,qty)
	
	#Print the values from the gesture sensor
	def print_gesture(self):
		data=self.paj7620ReadReg(0x43,1)[0]
		if data==self.GES_RIGHT_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				print("Forward")
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				print("Backward")
				time.sleep(self.GES_QUIT_TIME)
			else:
				print("Right")
				
		elif data==self.GES_LEFT_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				print("Forward")
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				print("Backward")
				time.sleep(self.GES_QUIT_TIME)
			else:
				print("Left")		
		
		elif data==self.GES_UP_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				print("Forward")
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				print("Backward")
				time.sleep(self.GES_QUIT_TIME)
			else:
				print("Up")		
				
		elif data==self.GES_DOWN_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				print("Forward")
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				print("Backward")
				time.sleep(self.GES_QUIT_TIME)
			else:
				print("Down")
		
		elif data==self.GES_FORWARD_FLAG:
			print("Forward")
			time.sleep(self.GES_QUIT_TIME)
			
		elif data==self.GES_BACKWARD_FLAG:
			print("Backward")
			time.sleep(self.GES_QUIT_TIME)
			
		elif data==self.GES_CLOCKWISE_FLAG:
			print("Clockwise")
	
		elif data==self.GES_COUNT_CLOCKWISE_FLAG:
			print("anti-clockwise")
			
		else:
			data1=self.paj7620ReadReg(0x44, 1)[0]
			if (data1 == self.GES_WAVE_FLAG):
				print("wave")
	
	#Return a vlaue from the gestire sensor which can be used in a program
	# 	0:nothing
	# 	1:Forward
	# 	2:Backward
	# 	3:Right
	# 	4:Left
	# 	5:Up
	# 	6:Down
	# 	7:Clockwise
	# 	8:anti-clockwise
	# 	9:wave
	def return_gesture(self):

		data=self.paj7620ReadReg(0x43,1)[0]
		if data==self.GES_RIGHT_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				return 1
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				return 2
				time.sleep(self.GES_QUIT_TIME)
			else:
				return 3
				
		elif data==self.GES_LEFT_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				return 1
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				return 2
				time.sleep(self.GES_QUIT_TIME)
			else:
				return 4	
		
		elif data==self.GES_UP_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				return 1
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				return 2
				time.sleep(self.GES_QUIT_TIME)
			else:
				return 5		
				
		elif data==self.GES_DOWN_FLAG:
			time.sleep(self.GES_ENTRY_TIME)
			data=self.paj7620ReadReg(0x43, 1)[0]
			if data == self.GES_FORWARD_FLAG:
				return 1
				time.sleep(self.GES_QUIT_TIME)
			elif data == self.GES_BACKWARD_FLAG:
				return 2
				time.sleep(self.GES_QUIT_TIME)
			else:
				return 6
		
		elif data==self.GES_FORWARD_FLAG:
			return 1
			time.sleep(self.GES_QUIT_TIME)
			
		elif data==self.GES_BACKWARD_FLAG:
			return 2
			time.sleep(self.GES_QUIT_TIME)
			
		elif data==self.GES_CLOCKWISE_FLAG:
			return 7

		elif data==self.GES_COUNT_CLOCKWISE_FLAG:
			return 8
			
		else:
			data1=self.paj7620ReadReg(0x44, 1)[0]
			if (data1 == self.GES_WAVE_FLAG):
				return 9
		return 0
		
def main():
    g = gesture()
    g.init()

    while True:
        g.print_gesture()
        time.sleep(.1)
        # print g.return_gesture()
        # time.sleep(.1)

if __name__ == '__main__':
    main()




"""
License
The MIT License (MIT)

Copyright (C) 2020 Ville Laine, Aikuiskoulutus Taitaja

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import serial
from time import sleep
import datetime


class GPS:
    def __init__(self, port='/dev/ttyAMA0', baud=9600, timeout=1):
        # Initialize the serial line
        # Serial line timeout has to be set because use of the readline()-function, otherwise can block forever
        self.serial = serial.Serial(port, baud, timeout=timeout)
        self.serial.flush()
        self.line = []

        # Define variables
        # Derived variables from message variables
        # gps time and date to Python datetime
        self.utc_datetime = datetime.datetime.now()
        # Full location string as Degrees and Decimal Minutes(Most commonly used format)
        self.location_DDM = ""

        # __GGA message has
        self.GGA_utc_time = self.utc_datetime.strftime("%H%M%S.%f")
        self.GGA_latitude = ""
        self.GGA_NS_indicator = ""
        self.GGA_longitude = ""
        self.GGA_EW_indicator = ""
        self.GGA_position_fix_indicator = ""
        self.GGA_fix_indicator = {0: "Fix not available or invalid",
                                  1: "gps SPS Mode, fix valid",
                                  2: "Differential gps, SPS Mode, fix valid",
                                  3: "Not supported",
                                  4: "Not supported",
                                  5: "Not supported",
                                  6: "Dead Reckoning Mode, fix valid"}
        self.GGA_satellites_used = ""
        self.GGA_HDOP = ""
        self.GGA_MSL_altitude = ""
        self.GGA_units1 = ""
        self.GGA_geoid_separation = ""
        self.GGA_units2 = ""
        self.GGA_age_of_diff_corr = ""
        self.GGA_diff_ref_stat_id = ""

        # __GSA message has
        self.GSA_mode1 = ""
        self.GSA_mode2 = ""
        self.GSA_satellite_used = ["", "", "", ""] * 12
        self.GSA_PDOP = ""
        self.GSA_HDOP = ""
        self.GSA_VDOP = ""

        # __GSV message has
        self.GSV_number_of_messages = 0
        self.GSV_message_number = 0
        self.GSV_satellites_in_view = 0
        self.GSV_satellite_data = [[""]]  # Size modified in __GSV base

        # __RMC message has
        self.RMC_utc_time = self.utc_datetime.strftime("%H%M%S.%f")
        self.RMC_status = ""
        self.RMC_latitude = ""  # same as __GGA
        self.RMC_NS_indicator = ""  # same as __GGA
        self.RMC_longitude = ""  # same as __GGA
        self.RMC_EW_indicator = ""  # same as __GGA
        self.RMC_speed_over_ground = ""
        self.RMC_course_over_ground = ""
        self.RMC_date = self.utc_datetime.strftime("%d%m%y")
        self.RMC_magnetic_variation = ""
        self.RMC_mv_EW_indicator = ""
        self.RMC_mode = ""

    # Check if serial interface buffer has unread bytes
    def new_data(self):
        if self.serial.in_waiting:
            return 1
        else:
            return 0

    def read(self):
        try:
            # If there is something to read then proceed
            if self.new_data():
                # Read one line of data
                if self.__read_line(3):
                    # Get the type without the sender to support all constellations
                    message_type = self.line[0][-3:]
                    # Select correct function to proceed if read successful
                    if message_type == "GGA":
                        return self.__GGA(self.line)
                    elif message_type == "GSA":
                        return self.__GSA(self.line)
                    elif message_type == "GSV":
                        return self.__GSV(self.line)
                    elif message_type == "RMC":
                        return self.__RMC(self.line)
        except IndexError:
            pass

        return 0

    def __read_line(self, tries):
        """
        Attempts n times at most to get valid data from gps
        Returns as soon as valid data is found
        If data read fails, then return empty list to self.line variable
        """
        for _ in range(tries):
            try:
                # Read one line from serial
                # Decode it to utf-8-format
                line = self.serial.readline().decode('utf-8')
                # Add comma before checksum *
                index = line.find("*")
                line = line[:index] + "," + line[index:]

                # Strip leading and trailing spaces and
                # Convert comma separated string to list with split
                self.line = line.strip().split(",")
                return 1
            except Exception:
                pass

        # Should never reach here if everything is ok
        self.line = []
        return 0

    #############################################
    # Message ID __GGA ($GPGGA)                 #
    # Global Positioning System Fixed Data      #
    #############################################
    def __GGA(self, line):
        if self.__check_checksum(line):
            self.GGA_utc_time = line[1]
            self.GGA_latitude = line[2]
            self.GGA_NS_indicator = line[3]
            self.GGA_longitude = line[4]
            self.GGA_EW_indicator = line[5]
            self.GGA_position_fix_indicator = line[6]
            self.GGA_satellites_used = line[7]
            self.GGA_HDOP = line[8]
            self.GGA_MSL_altitude = line[9]
            self.GGA_units1 = line[10]
            self.GGA_geoid_separation = line[11]
            self.GGA_units2 = line[12]
            self.GGA_age_of_diff_corr = line[13]
            self.GGA_diff_ref_stat_id = line[14]

            self.__update_datetime()
            self.__update_location("__GGA")
            return 1
        else:
            # print("Checksum failed!")
            return 0

    #############################################
    # Message ID __GSA ($GPGSA)                 #
    # GNSS DOP and Active Satellites            #
    #############################################
    def __GSA(self, line):
        if self.__check_checksum(line):
            # print("GSA Checksum OK!")
            self.GSA_mode1 = line[1]
            self.GSA_mode2 = line[2]
            for i in range(0, 12):
                self.GSA_satellite_used[i] = line[i+3]
            self.GSA_PDOP = line[16]
            self.GSA_HDOP = line[17]
            self.GSA_VDOP = line[18]

            return 1
        else:
            # print("GSA Checksum failed!")
            return 0

    #############################################
    # Message ID __GSV ($GPGSV)                 #
    # GNSS Satellites in View                   #
    #############################################
    def __GSV(self, line):
        if self.__check_checksum(line):
            self.GSV_number_of_messages = int(line[1])
            self.GSV_message_number = int(line[2])
            self.GSV_satellites_in_view = int(line[3])

            # Modify satellite data variable size depend on number of reported satellites in messages
            # Only when the first message of the message bunch has arrived
            if self.GSV_message_number == 1:
                self.GSV_satellite_data = [["", "", "", ""]]*self.GSV_number_of_messages*4

            range_end = round((len(line)-5)/4)
            for i in range(0, range_end):
                self.GSV_satellite_data[i + 4*(self.GSV_message_number-1)] = [line[4+i*4],
                                                                              line[5+i*4],
                                                                              line[6+i*4],
                                                                              line[7+i*4]]
            return 1
        else:
            return 0

    #############################################
    # Message ID __RMC ($GPRMC)                 #
    # Recommended Minimum Specific GNSS Data    #
    #############################################
    def __RMC(self, line):
        if self.__check_checksum(line):
            self.RMC_utc_time = line[1]
            self.RMC_status = line[2]
            self.RMC_latitude = line[3]
            self.RMC_NS_indicator = line[4]
            self.RMC_longitude = line[5]
            self.RMC_EW_indicator = line[6]
            self.RMC_speed_over_ground = line[7]
            self.RMC_course_over_ground = line[8]
            self.RMC_date = line[9]
            self.RMC_magnetic_variation = line[10]
            self.RMC_mv_EW_indicator = line[11]
            self.RMC_mode = line[12]

            self.__update_datetime()
            self.__update_location("__RMC")
            return 1
        else:
            return 0

    # get Python datetime format from message strings
    def __update_datetime(self):
        # hhmmss.ss
        # which received time is newer
        time = self.GGA_utc_time if self.GGA_utc_time > self.RMC_utc_time else self.RMC_utc_time
        self.utc_datetime = datetime.datetime.strptime(self.RMC_date + time, "%d%m%y%H%M%S.%f")

    # self.position_DDM
    def __update_location(self, message):

        if message == "__GGA":
            self.location_DDM = (self.GGA_latitude[:2] + chr(176) +
                                 self.GGA_latitude[2:] + "'" +
                                 self.GGA_NS_indicator +
                                 self.GGA_longitude[:2] + chr(176) +
                                 self.GGA_longitude[2:] + "'" +
                                 self.GGA_EW_indicator)
        elif message == "__RMC":
            self.location_DDM = (self.RMC_latitude[:2] + chr(176) +
                                 self.RMC_latitude[2:] + "'" +
                                 self.RMC_NS_indicator +
                                 self.RMC_longitude[:2] + chr(176) +
                                 self.RMC_longitude[2:] + "'" +
                                 self.RMC_EW_indicator)

        if len(self.location_DDM) <= 4:
            self.location_DDM = "Unavailable"

    ########################################################################################
    # Like name says, calculates message checksum and compares it to the messages checksum #
    ########################################################################################
    @staticmethod
    def __check_checksum(line):
        checksum = 0x0

        stringline = line[0]
        for i in range(1, len(line)):
            # Do not add comma before *checksum
            if i is not len(line)-1:
                stringline += ","
            stringline += line[i]

        for char in stringline:
            if char == "$":
                # Pass the starting character
                pass
            elif char == "*":
                # Stop XOR'in when '*' found
                break
            else:
                # XOR character to checksum
                checksum ^= ord(char)

        # Modify checksum to match the NMEA format
        checksum = ("*" + str(hex(checksum))[2:]).upper()
        if len(checksum) < 3:
            # Add missing 0 to checksum string
            checksum = checksum[:1] + "0" + checksum[1:]

        # Return 1 if checksum matches the line checksum
        if checksum == line[len(line)-1]:
            return 1
        else:
            return 0



#!/usr/bin/python
#
# This is the library for Grove Base Hat.
#
# Factory Class
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import print_function
import sys
from grove.button      import *
from grove.led         import *
from grove.temperature import *
from grove.gpio        import *
from grove.display     import *
from grove.motor       import *

# GPIOWrapper settings
_wrapper_pir_motion = {
    'high-enable' : True,
    'direction'   : GPIO.IN,
    'status-attr' : "has_motion",
    'enable-attr' : "nothing",
    'disable-attr': "nothing"
}

_wrapper_buzzer = {
    'high-enable' : True,
    'direction'   : GPIO.OUT,
    'status-attr' : "sounding",
    'enable-attr' : "on",
    'disable-attr': "off"
}

_wrapper_electromagnet = {
    'high-enable' : True,
    'direction'   : GPIO.OUT,
    'status-attr' : "nothing",
    'enable-attr' : "on",
    'disable-attr': "off"
}

_wrapper_relay = {
    'high-enable' : True,
    'direction'   : GPIO.OUT,
    'status-attr' : "nothing",
    'enable-attr' : "on",
    'disable-attr': "off"
}

_stepper_motor_28BYJ48 = {
    'name'        : "28BYJ48",
    'var-ratio'   : 64,
    'stride-angle': 5.625,
    'rpm-max'     : 12,
    'sequences'   :
        [ 0b0001, 0b0011, 0b0010, 0b0110, 0b0100, 0b1100, 0b1000, 0b1001 ]
}

_stepper_motor_24BYJ48 = {
    'name'        : "24BYJ48",
    'var-ratio'   : 32,
    'stride-angle': 5.625,
    'rpm-max'     : 30,
    'sequences'   :
        [ 0b0001, 0b0011, 0b0010, 0b0110, 0b0100, 0b1100, 0b1000, 0b1001 ]
}

_stepper_motor_YH42BYGH40 = {
    'name'        : "YH42BYGH40",
    'var-ratio'   : 1.0,
    'stride-angle': 0.9,
    'rpm-max'     : 250,
    'sequences'   :
        [ 0b0001, 0b0101, 0b0100, 0b0110, 0b0010, 0b1010, 0b1000, 0b1001 ]
}

class __factory(object):
    ButtonEnum = ('Button', "GPIO-LOW", "GPIO-HIGH", "I2C", "I2C-POLL")
    OneLedEnum = ('OneLed', "GPIO-LOW", "GPIO-HIGH", "WS2812-PWM")
    TemperEnum = ('Temper', "NTC-ADC",  "MCP9808-I2C")
    GPIOWrapperEnum = ('GPIOWrapper', "PIRMotion", "Buzzer", "Electromagnet", "Relay")
    StepperMotorEnum = ('StepperMotor', "28BYJ48", "24BYJ48", "YH42BYGH40")
    DisplayEnum = ('Display', "JHD1802", "SH1107G")

    def __init__(self):
        pass

    def __avail_list(self, typ, enum):
        print("Factory.get: Wrong {} type specified {}".format(enum[0], typ))
        print("Available types: ", end='')
        for name in enum[1:]:
            print(name, ',', sep='', end='')
        print("\b ")

    def getButton(self, typ, pin):
        if typ == "GPIO-LOW":
            return ButtonTypedGpio(pin, True)
        elif typ == "GPIO-HIGH":
            return ButtonTypedGpio(pin, False)
        elif typ == "I2C":
            return ButtonTypedI2c()
        elif typ == "I2C-POLL":
            return ButtonTypedI2c(evt_en = False)
        else:
            self.__avail_list(typ, self.ButtonEnum)
            sys.exit(1)

    def getOneLed(self, typ, pin):
        if typ == "GPIO-LOW":
            return OneLedTypedGpio(pin, False)
        elif typ == "GPIO-HIGH":
            return OneLedTypedGpio(pin, True)
        elif typ == "WS2812-PWM":
            from grove.led.one_led_ws2812 import OneLedTypedWs2812
            return OneLedTypedWs2812(pin)
        else:
            self.__avail_list(typ, self.OneLedEnum)
            sys.exit(1)

    def getTemper(self, typ, channel = None, address=0x18):
        if typ == "NTC-ADC":
            return TemperTypedNTC(channel)
        elif typ == "MCP9808-I2C":
            return TemperMCP9808(address=address)
        else:
            self.__avail_list(typ, self.TemperEnum)
            sys.exit(1)

    def getGpioWrapper(self, typ, pin):
        if typ == "PIRMotion":
            return GPIOWrapper(pin, _wrapper_pir_motion)
        elif typ == "Buzzer":
            return GPIOWrapper(pin, _wrapper_buzzer)
        elif typ == "Electromagnet":
            return GPIOWrapper(pin, _wrapper_electromagnet)
        elif typ == "Relay":
            return GPIOWrapper(pin, _wrapper_relay)
        else:
            self.__avail_list(typ, self.GPIOWrapperEnum)
            sys.exit(1)

    def getStepperMotor(self, typ):
        if typ == "28BYJ48":
            return I2CStepperMotor(_stepper_motor_28BYJ48)
        elif typ == "24BYJ48":
            return I2CStepperMotor(_stepper_motor_24BYJ48)
        elif typ == "YH42BYGH40":
            return I2CStepperMotor(_stepper_motor_YH42BYGH40)
        else:
            self._avail_list(typ, self.StepperMotorEnum)
            sys.exit(1)

    def getDisplay(self, typ):
        if typ == "JHD1802":
            return JHD1802()
        elif typ == "SH1107G":
            return SH1107G()
        else:
            self._avail_list(typ, self.DisplayEnum)
            sys.exit(1)

    # Compability
    getLcd = getDisplay



Factory = __factory()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is the code for
    - `Grove - 16 x 2 LCD (Black on Red) <https://www.seeedstudio.com/Grove-16-x-2-LCD-Black-on-Re-p-3197.html>`_
    - `Grove - 16 x 2 LCD (Black on Yellow) <https://www.seeedstudio.com/Grove-16-x-2-LCD-Black-on-Yello-p-3198.html>`_
    - `Grove - 16 x 2 LCD (White on Blue) <https://www.seeedstudio.com/Grove-16-x-2-LCD-White-on-Blu-p-3196.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.factory import Factory

        # LCD 16x2 Characters
        lcd = Factory.getDisplay("JHD1802")
        rows, cols = lcd.size()
        print("LCD model: {}".format(lcd.name))
        print("LCD type : {} x {}".format(cols, rows))

        lcd.setCursor(0, 0)
        lcd.write("hello world!")
        lcd.setCursor(0, cols - 1)
        lcd.write('X')
        lcd.setCursor(rows - 1, 0)
        for i in range(cols):
            lcd.write(chr(ord('A') + i))

        time.sleep(3)
        lcd.clear()
'''

from grove.display.base import *
from grove.i2c import Bus
import time
import sys

# sphinx autoapi required
__all__ = ["JHD1802"]

class JHD1802(Display):
    '''
    Grove - 16 x 2 LCD, using chip JHD1802.
        - Grove - 16 x 2 LCD (Black on Yellow)
        - Grove - 16 x 2 LCD (Black on Red)
        - Grove - 16 x 2 LCD (White on Blue)

    Also, it's our class name,
    which could drive the above three LCDs.

    Args:
        address(int): I2C device address, default to 0x3E.
    '''
    def __init__(self, address = 0x3E):
        self._bus = Bus()
        self._addr = address
        if self._bus.write_byte(self._addr, 0):
            print("Check if the LCD {} inserted, then try again"
                    .format(self.name))
            sys.exit(1)
        self.textCommand(0x02)
        time.sleep(0.1)
        self.textCommand(0x08 | 0x04) # display on, no cursor
        self.textCommand(0x28)

    @property
    def name(self):
        '''
        Get device name

        Returns:
            string: JHD1802
        '''
        return "JHD1802"

    def type(self):
        '''
        Get device type

        Returns:
            int: ``TYPE_CHAR``
        '''
        return TYPE_CHAR

    def size(self):
        '''
        Get display size

        Returns:
            (Rows, Columns): the display size, in characters.
        '''
        # Charactor 16x2
        # return (Rows, Columns)
        return 2, 16

    def clear(self):
        '''
        Clears the screen and positions the cursor in the upper-left corner.
        '''
        self.textCommand(0x01)

    def draw(self, data, bytes):
        '''
        Not implement for char type display device.
        '''
        return False

    def home(self):
        '''
        Positions the cursor in the upper-left of the LCD.
        That is, use that location in outputting subsequent text to the display.
        '''
        self.textCommand(0x02)
        time.sleep(0.2)

    def setCursor(self, row, column):
        '''
        Position the LCD cursor; that is, set the location
        at which subsequent text written to the LCD will be displayed.

        Args:
            row   (int): the row at which to position cursor, with 0 being the first row
            column(int): the column at which to position cursor, with 0 being the first column

	Returns:
	    None
        '''
        # print("setCursor: row={}, column={}".format(row,column))
        self.textCommand((0x40 * row) + (column % 0x10) + 0x80)

    def write(self, msg):
        '''
        Write character(s) to the LCD.

        Args:
            msg (string): the character(s) to write to the display

        Returns:
            None
        '''
        for c in msg:
            self._bus.write_byte_data(self._addr,0x40,ord(c))

    def _cursor_on(self, enable):
        if enable:
            self.textCommand(0x0E)
        else:
            self.textCommand(0x0C)
            
            
    def textCommand(self, cmd):
        self._bus.write_byte_data(self._addr,0x80,cmd)
        
def main():
    import time

    lcd = JHD1802()
    rows, cols = lcd.size()
    print("LCD model: {}".format(lcd.name))
    print("LCD type : {} x {}".format(cols, rows))

    lcd.backlight(False)
    time.sleep(1)

    lcd.backlight(True)
    lcd.setCursor(0, 0)
    lcd.write("hello world!")
    lcd.setCursor(0, cols - 1)
    lcd.write('X')
    lcd.setCursor(rows - 1, 0)
    for i in range(cols):
        lcd.write(chr(ord('A') + i))

    time.sleep(3)
    lcd.clear()

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Red LED    <https://www.seeedstudio.com/Grove-Red-LED-p-1142.html>`_
    - `Grove - Green LED  <https://www.seeedstudio.com/Grove-Green-LED-p-1144.html>`_
    - `Grove - Purple LED <https://www.seeedstudio.com/Grove-Purple-LED-3m-p-1143.html>`_
    - `Grove - White LED  <https://www.seeedstudio.com/Grove-White-LED-p-1140.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_led import GroveLed

        # connect to pin 5(slot D5)
        PIN   = 5
        led = GroveLed(PIN)

        while True:
            led.on()
            time.sleep(1)
            led.off()
            time.sleep(1)
'''
import time
from grove.gpio import GPIO

__all__ = ['GroveLed', 'GPIO']

class GroveLed(GPIO):
    '''
    Class for Grove - XXXX Led

    Args:
        pin(int): number of digital pin the led connected.
    '''
    def __init__(self, pin):
        super(GroveLed, self).__init__(pin, GPIO.OUT)

    def on(self):
        '''
        light on the led
        '''
        self.write(1)

    def off(self):
        '''
        light off the led
        '''
        self.write(0)


Grove = GroveLed


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    led = GroveLed(pin)

    while True:
        led.on()
        time.sleep(1)
        led.off()
        time.sleep(1)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
'''
This is the code for
    - `Grove - Touch Sensor <https://www.seeedstudio.com/Grove-Touch-Sensor-p-747.html>`_

Examples:

    .. code-block:: python

        from grove.grove_touch_sensor import GroveTouchSensor
        import time

        # connect to pin 5 (slot D5)
        PIN = 5
        touch = GroveTouchSensor(PIN)

        def on_press(t):
            print('Pressed')
        def on_release(t):
            print("Released.")

        touch.on_press = on_press
        touch.on_release = on_release

        while True:
            time.sleep(1)
'''
import time
from grove.gpio import GPIO

__all__ = ['GroveTouchSensor']

class GroveTouchSensor(GPIO):
    '''
    Grove Touch Sensor class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin):
        super(GroveTouchSensor, self).__init__(pin, GPIO.IN)
        self._last_time = time.time()

        self._on_press = None
        self._on_release = None

    @property
    def on_press(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a button pressing.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_press = callback

            get

            .. code-block:: python

                callobj = obj.on_press
        '''
        return self._on_press

    @on_press.setter
    def on_press(self, callback):
        if not callable(callback):
            return

        if self.on_event is None:
            self.on_event = self._handle_event

        self._on_press = callback

    @property
    def on_release(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a button releasing.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_release = callback

            get

            .. code-block:: python

                callobj = obj.on_release
        '''
        return self._on_release

    @on_release.setter
    def on_release(self, callback):
        if not callable(callback):
            return

        if self.on_event is None:
            self.on_event = self._handle_event

        self._on_release = callback

    def _handle_event(self, pin, value):
        t = time.time()
        dt, self._last_time = t - self._last_time, t

        if value:
            if callable(self._on_press):
                self._on_press(dt)
        else:
            if callable(self._on_release):
                self._on_release(dt)

Grove = GroveTouchSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    touch = GroveTouchSensor(pin)

    def on_press(t):
        print('Pressed')
    def on_release(t):
        print("Released.")

    touch.on_press = on_press
    touch.on_release = on_release

    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Library for Grove - Temperature & Humidity Sensor (SHT31)
# (https://www.seeedstudio.com/Grove-Temperature-Humidity-Sensor-SHT3-p-2655.html)
#

'''
## License

The MIT License (MIT)

Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
from grove.i2c import Bus


def CRC(data):
    crc = 0xff
    for s in data:
        crc ^= s
        for _ in range(8):
            if crc & 0x80:
                crc <<= 1
                crc ^= 0x131
            else:
                crc <<= 1
    return crc


class GroveTemperatureHumiditySensorSHT3x(object):

    def __init__(self, address=0x44, bus=None):
        self.address = address

        # I2C bus
        self.bus = Bus(bus)

    def read(self):
        # high repeatability, clock stretching disabled
        self.bus.write_i2c_block_data(self.address, 0x24, [0x00])

        # measurement duration < 16 ms
        time.sleep(0.016)

        # read 6 bytes back
        # Temp MSB, Temp LSB, Temp CRC, Humididty MSB, Humidity LSB, Humidity CRC
        data = self.bus.read_i2c_block_data(self.address, 0x00, 6)

        if data[2] != CRC(data[:2]):
            raise ValueError("temperature CRC mismatch")
        if data[5] != CRC(data[3:5]):
            raise ValueError("humidity CRC mismatch")


        temperature = data[0] * 256 + data[1]
        celsius = -45 + (175 * temperature / 65535.0)
        humidity = 100 * (data[3] * 256 + data[4]) / 65535.0

        return celsius, humidity


Grove = GroveTemperatureHumiditySensorSHT3x


def main():
    sensor = GroveTemperatureHumiditySensorSHT3x()
    while True:
        temperature, humidity = sensor.read()

        print('Temperature in Celsius is {:.2f} C'.format(temperature))
        print('Relative Humidity is {:.2f} %'.format(humidity))

        time.sleep(1)


if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is Helper Classes

'''
from __future__ import print_function
from grove.adc import *
import time
import sys
import os
import re
import io

_SlotsGPIORpi     = { 5:"D5", 12:"PWM", 16:"D16", 18:"D18", 22:"D22", 24:"D24", 26:"D26" }
_SlotsGPIORpiZero = { 5:"D5", 12:"PWM", 16:"D16"           }
_SlotsADCRpi      = { 0:"A0",  2:"A2",   4:"A4",   6:"A6"  }
_SlotsADCRpiZero  = { 0:"A0",  2:"A2",   4:"A4"            }
_SlotsPWMRpi      = {         12:"PWM",           18:"D18" }
_SlotsPWMRpiZero  = {         12:"PWM"                     }
_SlotsNull        = { }

__all__ = ['SlotHelper', 'OverlayHelper']

class SlotHelper(object):
    # Slot types
    GPIO = 0
    ADC  = 1
    PWM  = 2
    I2C  = 3
    UART = 4

    # Platform types
    PLATFORM_UNKOWN      = 0
    PLATFORM_RPI         = 1
    PLATFORM_RPI_ZERO    = 2
    PLATFORM_CORAL       = 3
    PLATFORM_JETSON_NANO = 4

    def __init__(self, slot = GPIO):
        adc = ADC()
        self.name = adc.name
        print("Hat Name = '{}'".format(self.name))

        self.plat = self.get_platform()

        if self.name == RPI_ZERO_HAT_NAME:
            self.__hat_type = RPI_ZERO_HAT_PID
            self.__slots_gpio = _SlotsGPIORpiZero
            self.__slots_adc  = _SlotsADCRpiZero
            self.__slots_pwm  = _SlotsPWMRpiZero
        else:
            if self.name != RPI_HAT_NAME:
                print("Unknown hat, assume {}".format(RPI_HAT_NAME))
            self.__hat_type = RPI_HAT_PID
            self.__slots_gpio = _SlotsGPIORpi
            self.__slots_adc  = _SlotsADCRpi
            self.__slots_pwm  = _SlotsPWMRpi
        self.__slots_i2c = _SlotsNull

        # fix support for specific platform
        if self.plat == self.PLATFORM_CORAL:
            self.__slots_gpio.pop(12)
            self.__slots_gpio.pop(18)
            self.__slots_gpio.pop(22)

            self.__slots_pwm.pop(18)
            self.__slots_pwm[22] = 'D22'

        maps = {                       \
                SlotHelper.GPIO:self.__slots_gpio, \
                SlotHelper.ADC :self.__slots_adc,  \
                SlotHelper.PWM :self.__slots_pwm,  \
                SlotHelper.I2C :self.__slots_i2c,  \
                }

        self._slots = maps.get(slot)
        self._slot  = slot

    def is_adapted(self, pin):
        if not self._slots:
            return False
        if not pin in self._slots.keys():
            return False
        return True

    def list_avail(self):
        if not self._slots:
            return

        maps = {                          \
                SlotHelper.GPIO: "GPIO",  \
                SlotHelper.ADC : "ADC",   \
                SlotHelper.PWM : "PWM",   \
                SlotHelper.I2C : "I2C",   \
                }

        print(" <pin> could be one of below values")
        print("       in the pin column for {} function".format(maps.get(self._slot)))
        print("   And connect the device to corresponding slot")
        print("==============")
        print(" pin | slot")
        print("==============")
        for pin, slot in self._slots.items():
            print('{:^5}|{:^5} '.format(pin, slot))

    def argv2pin(self, extra=''):
        if len(sys.argv) < 2:
            usage = 'Usage: {} <pin>'.format(sys.argv[0])
            usage += extra
            print(usage)
            self.list_avail()
            sys.exit(1)

        pin = int(sys.argv[1])
        if not self.is_adapted(pin):
            self.list_avail()
            sys.exit(1)
        return pin

    def get_platform(self):
        plat = self.PLATFORM_UNKOWN
        model = io.open("/proc/device-tree/model").read().strip()
        if   re.match(r"^Raspberry Pi.*", model) is not None:
            plat = self.PLATFORM_RPI_ZERO if self.name == RPI_ZERO_HAT_NAME else self.PLATFORM_RPI
        elif re.match(r"^Freescale i.MX8MQ Phanbell.*", model) is not None:
            plat = self.PLATFORM_CORAL
        return plat

def root_check():
    if os.geteuid() != 0:
        print("This program must be run as root.")
        print("sudo required for non-root user, Aborting.")
        sys.exit(1)

def __module_installed(name):
    for line in os.popen("lsmod"):
        match = re.match("^" + name + " *", line)
        if match is None: continue
        # print("result = {}".format(match))
        return True
    return False

def module_install(name, param):
    if __module_installed(name):
        return True
    os.system("modprobe " + name + " " + param)
    for _ in range(20):
        if __module_installed(name):
            return True
        time.sleep(0.2)
    return False


class OverlayHelper(object):
    def __init__(self, dev_path, overlay, param):
        self._path = dev_path
        self._ovlay = overlay
        self._param = param

    def __is_dt_inst(self):
        for line in os.popen("dtoverlay -l"):
            # lines likes
            #2:  w1-gpio  gpiopin=5
            match = re.match("^[0-9]+: *" + self._ovlay + " +", line)
            if match is None: continue
            return True
        return False

    def is_installed(self):
        if os.path.exists(self._path):
            return True
        if self.__is_dt_inst():
            return True
        return False

    def install(self):
        if self.is_installed():
            return True
        os.system("dtoverlay " + self._ovlay + " " + self._param)
        for _ in range(20):
            if self.is_installed():
                return True
            time.sleep(0.2)
        return False

    def __str__(self):
        return "Overlay {} installed = {}".format(
               self._ovlay, self.is_installed())

    # __repr__ = __str__

    @property
    def name(self):
        return self._ovlay


if __name__ == '__main__':
    helper = SlotHelper(SlotHelper.PWM)
    helper.list_avail()
    print("platform = {}".format(helper.plat))
    sys.exit(0)

    print("module w1_therm installed: {}"
          .format(__module_installed("w1_therm")))
    module_install("w1_therm", "")
    print("module w1_therm installed: {}"
          .format(__module_installed("w1_therm")))
    print("module w1_gpio  installed: {}"
          .format(__module_installed("w1_gpio")))
    oh = OverlayHelper("/sys/devices/w1_bus_master1",
                       "w1-gpio",
                       "gpiopin=5")
    print(oh)
    print("install {} ...".format(oh.name))
    oh.install()
    print(oh)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Thumb Joystick <https://www.seeedstudio.com/Grove-Thumb-Joystick-p-935.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_thumb_joystick import GroveThumbJoystick

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveThumbJoystick(PIN)

        while True:
            x, y = sensor.value
            if x > 900:
                print('Joystick Pressed')
            print("X, Y = {0} {1}".format(x, y))
            time.sleep(.2)
'''
import math
import sys
import time
from grove.adc import ADC

__all__ = ['GroveThumbJoystick']

class GroveThumbJoystick(object):
    '''
    Grove Thumb Joystick class

    Args:
        channel(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channelX = channel
        self.channelY = channel + 1
        self.adc = ADC()

    @property
    def value(self):
        '''
        Get the water strength value

        Returns:
            (pair): x-ratio, y-ratio, all are 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channelX), self.adc.read(self.channelY)

Grove = GroveThumbJoystick


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveThumbJoystick(pin)

    while True:
        x, y = sensor.value
        if x > 900:
            print('Joystick Pressed')
        print("X, Y = {0} {1}".format(x, y))
        time.sleep(.2)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the code for Grove - Ultrasonic Ranger
# (https://www.seeedstudio.com/Grove-Ultrasonic-Ranger-p-960.html)
# which is a non-contact distance measurement module which works with 40KHz sound wave. 
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import sys
import time
from grove.gpio import GPIO

usleep = lambda x: time.sleep(x / 1000000.0)

_TIMEOUT1 = 1000
_TIMEOUT2 = 10000

class GroveUltrasonicRanger(object):
    def __init__(self, pin):
        self.dio = GPIO(pin)

    def _get_distance(self):
        self.dio.dir(GPIO.OUT)
        self.dio.write(0)
        usleep(2)
        self.dio.write(1)
        usleep(10)
        self.dio.write(0)

        self.dio.dir(GPIO.IN)

        t0 = time.time()
        count = 0
        while count < _TIMEOUT1:
            if self.dio.read():
                break
            count += 1
        if count >= _TIMEOUT1:
            return None

        t1 = time.time()
        count = 0
        while count < _TIMEOUT2:
            if not self.dio.read():
                break
            count += 1
        if count >= _TIMEOUT2:
            return None

        t2 = time.time()

        dt = int((t1 - t0) * 1000000)
        if dt > 530:
            return None

        distance = ((t2 - t1) * 1000000 / 29 / 2)    # cm

        return distance

    def get_distance(self):
        while True:
            dist = self._get_distance()
            if dist:
                return dist


Grove = GroveUltrasonicRanger


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    sonar = GroveUltrasonicRanger(pin)

    print('Detecting distance...')
    while True:
        print('{} cm'.format(sonar.get_distance()))
        time.sleep(1)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
#
# This library is for Grove - 4 Digit Display(https://www.seeedstudio.com/Grove-Servo-p-1241.html) which has 4 red seven-segment displays
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import sys
import time
from grove.gpio import GPIO


charmap = {
    '0': 0x3f,
    '1': 0x06,
    '2': 0x5b,
    '3': 0x4f,
    '4': 0x66,
    '5': 0x6d,
    '6': 0x7d,
    '7': 0x07,
    '8': 0x7f,
    '9': 0x6f,
    'A': 0x77,
    'B': 0x7f,
    'b': 0x7C,
    'C': 0x39,
    'c': 0x58,
    'D': 0x3f,
    'd': 0x5E,
    'E': 0x79,
    'F': 0x71,
    'G': 0x7d,
    'H': 0x76,
    'h': 0x74,
    'I': 0x06,
    'J': 0x1f,
    'K': 0x76,
    'L': 0x38,
    'l': 0x06,
    'n': 0x54,
    'O': 0x3f,
    'o': 0x5c,
    'P': 0x73,
    'r': 0x50,
    'S': 0x6d,
    'U': 0x3e,
    'V': 0x3e,
    'Y': 0x66,
    'Z': 0x5b,
    '-': 0x40,
    '_': 0x08,
    ' ': 0x00
}

ADDR_AUTO = 0x40
ADDR_FIXED = 0x44
STARTADDR = 0xC0
BRIGHT_DARKEST = 0
BRIGHT_DEFAULT = 2
BRIGHT_HIGHEST = 7


class Grove4DigitDisplay(object):
    colon_index = 1

    def __init__(self, clk, dio, brightness=BRIGHT_DEFAULT):
        self.brightness = brightness

        self.clk = GPIO(clk, direction=GPIO.OUT)
        self.dio = GPIO(dio, direction=GPIO.OUT)
        self.data = [0] * 4
        self.show_colon = False

    def clear(self):
        self.show_colon = False
        self.data = [0] * 4
        self._show()

    def show(self, data):
        if type(data) is str:
            for i, c in enumerate(data):
                if c in charmap:
                    self.data[i] = charmap[c]
                else:
                    self.data[i] = 0
                if i == self.colon_index and self.show_colon:
                    self.data[i] |= 0x80
                if i == 3:
                    break
        elif type(data) is int:
            self.data = [0, 0, 0, charmap['0']]
            if data < 0:
                negative = True
                data = -data
            else:
                negative = False
            index = 3
            while data != 0:
                self.data[index] = charmap[str(data % 10)]
                index -= 1
                if index < 0:
                    break
                data = int(data / 10)

            if negative:
                if index >= 0:
                    self.data[index] = charmap['-']
                else:
                    self.data = charmap['_'] + [charmap['9']] * 3
        else:
            raise ValueError('Not support {}'.format(type(data)))
        self._show()

    def _show(self):
        with self:
            self._transfer(ADDR_AUTO)

        with self:
            self._transfer(STARTADDR)
            for i in range(4):
                self._transfer(self.data[i])

        with self:
            self._transfer(0x88 + self.brightness)

    def update(self, index, value):
        if index < 0 or index > 4:
            return

        if value in charmap:
            self.data[index] = charmap[value]
        else:
            self.data[index] = 0

        if index == self.colon_index and self.show_colon:
            self.data[index] |= 0x80

        with self:
            self._transfer(ADDR_FIXED)

        with self:
            self._transfer(STARTADDR | index)
            self._transfer(self.data[index])

        with self:
            self._transfer(0x88 + self.brightness)


    def set_brightness(self, brightness):
        if brightness > 7:
            brightness = 7

        self.brightness = brightness
        self._show()

    def set_colon(self, enable):
        self.show_colon = enable
        if self.show_colon:
            self.data[self.colon_index] |= 0x80
        else:
            self.data[self.colon_index] &= 0x7F
        self._show()

    def _transfer(self, data):
        for _ in range(8):
            self.clk.write(0)
            if data & 0x01:
                self.dio.write(1)
            else:
                self.dio.write(0)
            data >>= 1
            time.sleep(0.000001)
            self.clk.write(1)
            time.sleep(0.000001)

        self.clk.write(0)
        self.dio.write(1)
        self.clk.write(1)
        self.dio.dir(GPIO.IN)

        while self.dio.read():
            time.sleep(0.001)
            if self.dio.read():
                self.dio.dir(GPIO.OUT)
                self.dio.write(0)
                self.dio.dir(GPIO.IN)
        self.dio.dir(GPIO.OUT)

    def _start(self):
        self.clk.write(1)
        self.dio.write(1)
        self.dio.write(0)
        self.clk.write(0)

    def _stop(self):
        self.clk.write(0)
        self.dio.write(0)
        self.clk.write(1)
        self.dio.write(1)

    def __enter__(self):
        self._start()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._stop()


Grove = Grove4DigitDisplay


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    display = Grove4DigitDisplay(pin, pin + 1)

    count = 0
    while True:
        t = time.strftime("%H%M", time.localtime(time.time()))
        display.show(t)
        display.set_colon(count & 1)
        count += 1
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2020 Ville Laine, Aikuiskoulutus Taitaja
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from time import sleep
from grove.modules.bme280 import bme280

bme_sensor = bme280()

# Set oversampling
# bme280 class defines OVRS_x0, .._x1, .._x2, .._x4, .._x8, .._x16
# set_oversampling(osrs_h(humidity), osrs_t(temperature), osrs_p(pressure))
# Set_oversampling > OVRS_x0 to enable the measurement, OVRS_x0 disables the measurement
bme_sensor.set_oversampling(bme280.OVRS_x16, bme280.OVRS_x16, bme280.OVRS_x16)

# Set internal IIR filter coefficient. 0 = no filter
iir_filter = bme280.filter_16
bme_sensor.set_filter(iir_filter)

# Know values for pressure correction
current_level_from_sea = 103            # Know height from sealevel m
current_sea_level_pressure = 1027.5     # Forecast data: current pressure at sealevel
count = 0                               # Just for counting delays
calibration_set = 0                     # Help bit

while True:
    try:
        # set mode to FORCE that is one time measurement
        # bme280.MODE_SLEEP, ...FORCE, ...NORMAL
        # If normal mode also set t_sb that is standby time between measurements
        # if not specified is set to 1000ms bme280.t_sb_1000
        # Returns 1 on success 0 otherwise
        if not bme_sensor.set_mode(bme280.MODE_FORCE):
            print("\nMode change failed!")

        # Measure raw signals measurements are put in bme280.raw_* variables
        # Returns 1 on success otherwise 0
        if not bme_sensor.read_raw_signals():
            print("\nError in measurement!")

        # Compensate the raw signals
        if not bme_sensor.read_compensated_signals():
            print("\nError compensating values")

        # Set pressure calibration after couple measurements
        # Sensor need to settle after soft reset given in the beginning
        # Delay depends on IIR filter step response and the 90% response is 2**iir_filter*2 seconds
        response_time = (2**iir_filter)*2
        if count == response_time:
            bme_sensor.set_pressure_calibration(level=current_level_from_sea, pressure=current_sea_level_pressure)
            count = response_time + 1
            calibration_set = 1
            # Update the compensated values because new calibration value is given
            if not bme_sensor.read_compensated_signals():
                print("\nError compensating values")
            print("Sensor compensation is set")
        elif count < response_time:
            print("Wait for sensor to settle before setting compensation!", response_time-count, "s")
            count += 1

        # Only print out signals after calibration
        if calibration_set:
            # Only works if pressure calibration is done with set_pressure_calibration()
            altitude = bme_sensor.get_altitude(current_sea_level_pressure)

            # Print out the data
            print("Temperature: %.2f" % bme_sensor.temperature, chr(176) + "C")
            print("Pressure: %.2fhPa, where correction is %.2fhPa, sensor reading is %.2fhPa"
                  % (bme_sensor.calibrated_pressure, bme_sensor.calibration_pressure, bme_sensor.pressure))
            print("Humidity: %.2f" % bme_sensor.humidity, "%RH")
            print("altitude from sea level: %.3fm, %.3f" % (altitude, bme_sensor.calibrated_pressure + altitude/8))
            print("\n")

        # Sleep 1 second
        sleep(1)
    except KeyboardInterrupt:
        break



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd. 
#
'''
 This is the code for GPIO Button event detection.
'''
from __future__ import division
import time
import threading
from grove.button import Button
from grove.gpio import GPIO

__all__ = ["ButtonTypedGpio"]

_CYCLE_PERIOD    = 0.02   # 20 ms
_SINGLE_KEY_TM   = 0.03   # 30 ms
_KEY_INTERVAL    = 0.3    # 300 ms
_LONG_KEY_TM     = 2.0    # 2s

class ButtonTypedGpio(Button):
    '''
    GPIO Button Class

    provide event checking ability to derived class,
    should not use directly by end-user.
    The checking events include:

      - Button.EV_SINGLE_CLICK
      - Button.EV_DOUBLE_CLICK
      - Button.EV_LONG_PRESS
      - Button.EV_LEVEL_CHANGED

    Args:
        pin(int)         : GPIO pin number the button connected.
        low_pressed(bool): optional, default True

            True if the the button gpio level is low when pressed.

            False if level high when pressed
    '''
    # all key states in FSM
    KEY_STATE_IDLE    = 0
    KEY_STATE_DOWN    = 1
    KEY_STATE_ONESHOT = 2
    KEY_STATE_LONG    = 3

    def __init__(self, pin, low_pressed = True):
        super(ButtonTypedGpio, self).__init__(pin)

        self.__low_press = low_pressed

        self.__state = self.KEY_STATE_IDLE
        self.__duration = 0.0
        self.__distance = _KEY_INTERVAL
        self.__thrd_exit = False
        self.__thrd = None

        self.__gpio = GPIO(pin, GPIO.IN)
        self.__gpio.on_event = self.__gpio_event

        if self.__thrd is None or not self.__thrd.is_alive():
            self.__thrd = threading.Thread( \
                    target = ButtonTypedGpio.__thrd_chk_evt, \
                    args = (self,))
            self.__thrd.setDaemon(True)
            self.__thrd.start()

    def __del__(self):
        self.__thrd_exit = True
        while self.__thrd.isAlive():
            time.sleep(_CYCLE_PERIOD / _CYCLE_UNIT)
        self.__thrd.join()

    def is_pressed(self, index = 0):
        '''
        Get the button status if it's being pressed ?

        Args:
            index(int): optional, the arg `index` not be used.

        Returns:
            (bool): True  if the button is being pressed.
                    False if not.
        '''
        v = self.__gpio.read()
        return self.__low_press != bool(v)

    # called by GPIO library
    def __gpio_event(self, pin, value):
        press = self.is_pressed()
        tm = time.time()
        self._send_event(self.EV_LEVEL_CHANGED, press, tm)

    # key event FSM(finite state machine)
    def __key_evt_fsm(self, dt):
        r = 0
        press = self.is_pressed()
        self.__distance = self.__distance + dt

        if self.__state == self.KEY_STATE_IDLE:
            if press:
                self.__duration = 0.0
                self.__state = self.KEY_STATE_DOWN
        elif self.__state == self.KEY_STATE_DOWN:
            if press:
                self.__duration = self.__duration + dt
                if self.__duration >= _SINGLE_KEY_TM:
                    self.__state = self.KEY_STATE_ONESHOT
        elif self.__state == self.KEY_STATE_ONESHOT:
            if not press:
                # print("distance {}".format(self.__distance))
                if self.__distance >= _KEY_INTERVAL:
                    r = self.EV_SINGLE_CLICK
                else:
                    r = self.EV_DOUBLE_CLICK
            else:
                self.__duration = self.__duration + dt
                # print("duration {}".format(self.__duration))
                if self.__duration >= _LONG_KEY_TM:
                    r = self.EV_LONG_PRESS
                    self.__state = self.KEY_STATE_LONG
        elif self.__state == self.KEY_STATE_LONG:
            if not press:
                self.__distance = _KEY_INTERVAL

        if not press:
            self.__state = self.KEY_STATE_IDLE

        if r == self.EV_DOUBLE_CLICK:
            self.__distance = _KEY_INTERVAL
        elif r == self.EV_SINGLE_CLICK:
            self.__distance = 0.0

        return r, press


    # Thread to check events
    def __thrd_chk_evt(self):
        '''
        # prevent dither
        time.sleep(0.01)
        v = self.__gpio.read()
        if self.__low_press == bool(v):
            return
        '''
        self.__last_time = time.time();
        while not self.__thrd_exit:
        # or self.__state != self.KEY_STATE_IDLE:
            t = time.time()
            dt, self.__last_time = t - self.__last_time, t

            r, pressed = self.__key_evt_fsm(dt)
            if r:
                self._send_event(r, pressed, t)
            time.sleep(_CYCLE_PERIOD)



'''
## License

The MIT License (MIT)

Grove 8 channels 12 bit ADC Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
from grove.i2c import Bus

ADC_DEFAULT_IIC_ADDR = 0X04

ADC_CHAN_NUM = 8

REG_RAW_DATA_START = 0X10
REG_VOL_START = 0X20
REG_RTO_START = 0X30

REG_SET_ADDR = 0XC0


class Pi_hat_adc():
    def __init__(self,bus_num=1,addr=ADC_DEFAULT_IIC_ADDR):
        self.bus=Bus(bus_num)
        self.addr=addr

    
    #get all raw adc data,THe max value is 4095,cause it is 12 Bit ADC
    def get_all_adc_raw_data(self):
        array = []
        for i in range(ADC_CHAN_NUM):  
            data=self.bus.read_i2c_block_data(self.addr,REG_RAW_DATA_START+i,2)
            val=data[1]<<8|data[0]
            array.append(val)
        return array
    
    def get_nchan_adc_raw_data(self,n):
        data=self.bus.read_i2c_block_data(self.addr,REG_RAW_DATA_START+n,2)
        val =data[1]<<8|data[0]
        return val
    #get all data with unit mv.
    def get_all_vol_milli_data(self):
        array = []
        for i in range(ADC_CHAN_NUM):  
            data=self.bus.read_i2c_block_data(self.addr,REG_VOL_START+i,2)
            val=data[1]<<8|data[0]
            array.append(val)
        return array
    
    def get_nchan_vol_milli_data(self,n):
        data=self.bus.read_i2c_block_data(self.addr,REG_VOL_START+n,2)
        val =data[1]<<8|data[0]
        return val

    #get all data ratio,unit is 0.1%
    def get_all_ratio_0_1_data(self):
        array = []
        for i in range(ADC_CHAN_NUM):  
            data=self.bus.read_i2c_block_data(self.addr,REG_RTO_START+i,2)
            val=data[1]<<8|data[0]
            array.append(val)
        return array
    
    def get_nchan_ratio_0_1_data(self,n):
        data=self.bus.read_i2c_block_data(self.addr,REG_RTO_START+n,2)
        val =data[1]<<8|data[0]
        return val



ADC = Pi_hat_adc()
def main():
    raw_data=ADC.get_all_adc_raw_data()
    vol_data=ADC.get_all_vol_milli_data()
    ratio_data=ADC.get_all_ratio_0_1_data()
    print("raw data for each channel:(1-8chan)(12 bit-max=4096):")
    print(raw_data)
    print("voltage for each channel:(unit:mv,max=3300mv):")
    print(vol_data)
    print ("ratio for each channel(unit 0.1%,max=100.0%):")
    print(ratio_data)

    print(" ")
    print("NOTICE!!!:")
    print("The default setting of ADC PIN is floating_input.")
    print(" ")

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd. 
#
# This is the ADC library for Grove Base Hat
# which used to connect grove sensors for raspberry pi.
# 
'''
This is the code for
    - `Grove Base Hat for RPi      <https://www.seeedstudio.com/Grove-WS2813-RGB-LED-Strip-Waterproof-60-LED-m-1m-p-3126.html>`_
    - `Grove Base Hat for RPi Zero <https://www.seeedstudio.com/Grove-Base-Hat-for-Raspberry-Pi-Zero-p-3187.html>`_

Grove Base Hat incorparates a micro controller STM32F030F4.

Raspberry Pi does not have ADC unit, so we use an external chip
to transmit analog data to raspberry pi.

Examples:
    .. code-block:: python

        import time
        from grove.adc import ADC

        adc = ADC()
        while True:
            # Read channel 0(Slot A0) voltage
            print(adc.read_voltage(0))
            time.sleep(1)

'''
import sys
import grove.i2c

__all__ = [
    "ADC",
    "RPI_HAT_NAME", "RPI_ZERO_HAT_NAME",
    "RPI_HAT_PID", "RPI_ZERO_HAT_PID"
]

RPI_HAT_PID      = 0x0004
RPI_ZERO_HAT_PID = 0x0005
RPI_HAT_NAME     = 'Grove Base Hat RPi'
""" The HAT name to compare with return value of :class:`ADC.name` """
RPI_ZERO_HAT_NAME= 'Grove Base Hat RPi Zero'
""" The HAT name to compare with return value of :class:`ADC.name` """


class ADC(object):
    '''
    Class ADC for the ADC unit on Grove Base Hat for RPi.

    Args:
        address(int): optional, i2c address of the ADC unit, default 0x04
    '''
    def __init__(self, address = 0x08):
        self.address = address
        self.bus = grove.i2c.Bus()

    def read_raw(self, channel):
        '''
        Read the raw data of ADC unit, with 12 bits resolution.

        Args:
            channel (int): 0 - 7, specify the channel to read

        Returns:
            (int): the adc result, in [0 - 4095]
        '''
        addr = 0x10 + channel
        return self.read_register(addr)

    # read input voltage (mV)
    def read_voltage(self, channel):
        '''
        Read the voltage data of ADC unit.

        Args:
            channel (int): 0 - 7, specify the channel to read

        Returns:
            (int): the voltage result, in mV
        '''
        addr = 0x20 + channel
        return self.read_register(addr)

    # input voltage / output voltage (%)
    def read(self, channel):
        '''
        Read the ratio between channel input voltage and power voltage (most time it's 3.3V).

        Args:
            channel (int): 0 - 7, specify the channel to read

        Returns:
            (int): the ratio, in 0.1%
        '''
        addr = 0x30 + channel
        return self.read_register(addr)

    @property
    def name(self):
        '''
        Get the Hat name.

        Returns:
            (string): could be :class:`RPI_HAT_NAME` or :class:`RPI_ZERO_HAT_NAME`
        '''
        id = self.read_register(0x0)
        if id == RPI_HAT_PID:
            return RPI_HAT_NAME
        elif id == RPI_ZERO_HAT_PID:
            return RPI_ZERO_HAT_NAME

    @property
    def version(self):
        '''
        Get the Hat firmware version.

        Returns:
            (int): firmware version
        '''
        return self.read_register(0x2)

    # read 16 bits register
    def read_register(self, n):
        '''
        Read the ADC Core (through I2C) registers

        Grove Base Hat for RPI I2C Registers

            - 0x00 ~ 0x01: 
            - 0x10 ~ 0x17: ADC raw data
            - 0x20 ~ 0x27: input voltage
            - 0x29: output voltage (Grove power supply voltage)
            - 0x30 ~ 0x37: input voltage / output voltage

        Args:
            n(int): register address.

        Returns:
            (int) : 16-bit register value.
        '''
        try:
            self.bus.write_byte(self.address, n)
            return self.bus.read_word_data(self.address, n)
        except IOError:
            print("Check whether I2C enabled and   {}  or  {}  inserted".format \
                    (RPI_HAT_NAME, RPI_ZERO_HAT_NAME))
            sys.exit(2)
            return 0


if __name__ == '__main__':
    import time

    adc = ADC()
    while True:
        print(adc.read_voltage(0))
        time.sleep(1)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is the code for
    - `Grove - Red    LED Button <https://www.seeedstudio.com/Grove-Red-LED-Button-p-3096.html>`_
    - `Grove - Yellow LED Button <https://www.seeedstudio.com/Grove-Yellow-LED-Button-p-3101.html>`_
    - `Grove - Blue   LED Button <https://www.seeedstudio.com/Grove-Blue-LED-Button-p-3104.html>`_

Examples:
    .. code-block:: python

        from grove.button import Button
        import grove.grove_ryb_led_button.GroveLedButton
        import time

        # slot/gpio number your device plugin
        pin = 12
        obj = GroveLedButton(pin)

        # the default behavior of led is
        #   single click - on
        #   double click - blink
        #   long press   - off
        # remove \'\'\' pairs below to begin your experiment
        \'\'\'
        # define a customized event handle your self
        def cust_on_event(index, event, tm):
            # obj.led could be used to operate led
            print("event with code {}, time {}".format(event, tm))

        obj.on_event = cust_on_event
        \'\'\'
        while True:
            time.sleep(1)
'''

import time
from grove.button import Button
from grove.factory import Factory

# sphinx autoapi required
__all__ = ["GroveLedButton"]

class GroveLedButton(object):
    '''
    Grove Red/Yellow/Blue Led Button class

    all of them has a gpio button with low valid level of pressing,
    and a gpio led with high valid level for lighting.

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin):
        # High = light on
        self.led = Factory.getOneLed("GPIO-HIGH", pin)
        # Low = pressed
        self.btn = Factory.getButton("GPIO-LOW", pin + 1)
        self.__on_event = None
        self.btn.on_event(self, GroveLedButton.__handle_event)

    @property
    def on_event(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is button event
            callback prototype:
                callback(index, code, time)
            callback argument:
                Args:
                    index(int): button index, be in 0 to [button count - 1]

                    code (int): bits combination of
                              -  Button.EV_LEVEL_CHANGED
                              -  Button.EV_SINGLE_CLICK
                              -  Button.EV_DOUBLE_CLICK
                              -  Button.EV_LONG_PRESS

                    time(time): event generation time

                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_event = callback

            get

            .. code-block:: python

                callobj = obj.on_event
        '''
        return self.__on_event

    @on_event.setter
    def on_event(self, callback):
        if not callable(callback):
            return
        self.__on_event = callback

    def __handle_event(self, evt):
        # print("event index:{} event:{} pressed:{}"
        #       .format(evt['index'], evt['code'], evt['presesed']))
        if callable(self.__on_event):
            # the customized behavior
            self.__on_event(evt['index'], evt['code'], evt['time'])
            return

        # the default behavior
        self.led.brightness = self.led.MAX_BRIGHT
        event = evt['code']
        if event & Button.EV_SINGLE_CLICK:
            self.led.light(True)
            print("turn on  LED")
        elif event & Button.EV_DOUBLE_CLICK:
            self.led.blink()
            print("blink    LED")
        elif event & Button.EV_LONG_PRESS:
            self.led.light(False)
            print("turn off LED")




def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    ledbtn = GroveLedButton(pin)

    # remove ''' pairs below to begin your experiment
    '''
    # define a customized event handle your self
    def cust_on_event(index, event, tm):
        print("event with code {}, time {}".format(event, tm))

    ledbtn.on_event = cust_on_event
    '''
    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This library is for Grove - Buzzer(https://www.seeedstudio.com/Grove-Buzzer-p-768.html)
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#
# Author: Peter Yang <linsheng.yang@seeed.cc>
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# Author: Sarah Knepper <sarah.knepper@intel.com>
# Copyright (c) 2015 Intel Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
from __future__ import print_function

import time
import RPi.GPIO as GPIO

def main():
    from grove.helper import helper
    # helper.root_check()

    print("Insert Grove-Buzzer to Grove-Base-Hat slot PWM[12 13 VCC GND]")
    # Grove Base Hat for Raspberry Pi
    pin = 12
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(pin, GPIO.OUT)

    # create PWM instance
    pwm = GPIO.PWM(pin, 10)
    pwm.start(0) 

    chords = [1047, 1175, 1319, 1397, 1568, 1760, 1976]
    # Play sound (DO, RE, MI, etc.), pausing for 0.5 seconds between notes
    try:
        for note in chords:
            pwm.ChangeFrequency(note)
            pwm.ChangeDutyCycle(95)
            time.sleep(0.5) 
    finally:
        pwm.stop()
        GPIO.cleanup()

    print("Exiting application")

if __name__ == '__main__':
    main()


#!/usr/bin/env python
#
# GrovePi Library for using the Grove - I2C Motor Driver(http://www.seeedstudio.com/depot/Grove-I2C-Motor-Driver-p-907.html)
#
# The GrovePi connects the Raspberry Pi and Grove sensors.  You can learn more about GrovePi here:  http://www.dexterindustries.com/GrovePi
#
# Have a question about this library?  Ask on the forums here:  http://forum.dexterindustries.com/c/grovepi
#
# 2018-07-04 
# Modified by Seeed Studio, changed smbus module to smbus2

'''
## License

The MIT License (MIT)

GrovePi for the Raspberry Pi: an open source platform for connecting Grove Sensors to the Raspberry Pi.
Copyright (C) 2017  Dexter Industries

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import division
from grove.i2c import Bus
import time


class MotorDriver(object):
	__MotorSpeedSet             = 0x82
	__PWMFrequenceSet           = 0x84
	__DirectionSet              = 0xaa
	__MotorSetA                 = 0xa1
	__MotorSetB                 = 0xa5
	__Nothing                   = 0x01
	__EnableStepper             = 0x1a
	__UnenableStepper           = 0x1b
	__Stepernu                  = 0x1c
	I2CAddr                     = 0x0f  #Set the address of the I2CMotorDriver
	SPEED_MAX                   = 100

	def __init__(self,address=0x0f):
		self.I2CAddr = address
		self.bus = Bus()

	def __del__(self):
		self.set_speed(0, 0)

	#Maps speed from 0-100 to 0-255
	def _map_vals(self,value, leftMin, leftMax, rightMin, rightMax):
		#http://stackoverflow.com/questions/1969240/mapping-a-range-of-values-to-another
		# Figure out how 'wide' each range is
		leftSpan = leftMax - leftMin
		rightSpan = rightMax - rightMin

		# Convert the left range into a 0-1 range (float)
		valueScaled = float(value - leftMin) / float(leftSpan)

		# Convert the 0-1 range into a value in the right range.
		return int(rightMin + (valueScaled * rightSpan))
		
	#Set motor speed
	def set_speed(self, speed1 = 0, speed2 = 0):
		s1 = self._map_vals(speed1, 0, 100, 0, 255)
		s2 = self._map_vals(speed2, 0, 100, 0, 255)
		self.bus.write_i2c_block_data(self.I2CAddr, self.__MotorSpeedSet, [s1, s2])
		time.sleep(.02)
	
	#Set motor direction
	def set_dir(self, clock_wise1 = True, clock_wise2 = True):
		dir1 = 0b10 if clock_wise1 else 0b01
		dir2 = 0b10 if clock_wise2 else 0b01
		dir = (dir2 << 2) | dir1
		self.bus.write_i2c_block_data(self.I2CAddr, self.__DirectionSet, [dir, 0])
		time.sleep(.02)

Grove = MotorDriver

def main():
	print("Make sure I2C-Motor-Driver inserted")
	print("  in one I2C slot of Grove-Base-Hat")
	motor = MotorDriver()
	while True:
		motor.set_speed(MotorDriver.SPEED_MAX, MotorDriver.SPEED_MAX)
		motor.set_dir(True, True)
		time.sleep(2)
		motor.set_speed(MotorDriver.SPEED_MAX * 0.7, MotorDriver.SPEED_MAX * 0.7)
		motor.set_dir(False, False)
		time.sleep(2)

if __name__ == '__main__':
	main()




#!/usr/bin/env python
#
# Library for Grove - I2C Color Sensor V2(https://www.seeedstudio.com/Grove-I2C-Color-Sensor-V2-p-2890.html)
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
from grove.i2c import Bus


_CMD      = 0x80
_AUTO     = 0x20

_ENABLE   = 0x00
_ATIME    = 0x01
_WTIME    = 0x03
_AILT     = 0x04
_AIHT     = 0x06
_PERS     = 0x0C
_CONFIG   = 0x0D
_CONTROL  = 0x0F
_ID       = 0x12
_STATUS   = 0x13
_CDATA    = 0x14
_RDATA    = 0x16
_GDATA    = 0x18
_BDATA    = 0x1A

_AIEN       = 0x10
_WEN        = 0x08
_AEN        = 0x02
_PON        = 0x01

_GAINS  = (1, 4, 16, 60)


class GroveI2cColorSensorV2:
    """Driver for Grove I2C Color Sensor (TCS34725)"""

    def __init__(self, bus=None, address=0x29):
        self.address = address
        self.bus = Bus(bus)

        self.awake = False

        if self.id not in (0x44, 0x4D):
            raise ValueError('Not find a Grove I2C Color Sensor V2')

        self.set_integration_time(24)
        self.set_gain(4)

    def wakeup(self):
        enable = self._read_byte(_ENABLE)
        self._write_byte(_ENABLE, enable | _PON | _AEN)
        time.sleep(0.0024)

        self.awake = True

    def sleep(self):
        enable = self._read_byte(_ENABLE)
        self._write_byte(_ENABLE, enable & ~_PON)

        self.awake = False

    def is_awake(self):
        return self._read_byte(_ENABLE) & _PON

    def set_wait_time(self, t):
        pass

    @property
    def id(self):
        return self._read_byte(_ID)

    @property
    def integration_time(self):
        steps = 256 - self._read_byte(_ATIME)
        return steps * 2.4

    def set_integration_time(self, t):
        """Set the integration time of the sensor"""
        if t < 2.4:
            t = 2.4
        elif t > 614.4:
            t = 614.4
        
        steps = int(t / 2.4)
        self._integration_time = steps * 2.4
        self._write_byte(_ATIME, 256 - steps)

    @property
    def gain(self):
        """The gain control. Should be 1, 4, 16, or 60.
        """
        return _GAINS[self._read_byte(_CONTROL)]

    def set_gain(self, gain):
        if gain in _GAINS:
            self._write_byte(_CONTROL, _GAINS.index(gain))

    @property
    def raw(self):
        """Read RGBC registers
        return 16 bits red, green, blue and clear data
        """

        if not self.awake:
            self.wakeup()

        while not self._valid():
            time.sleep(0.0024)

        data = tuple(self._read_word(reg) for reg in (_RDATA, _GDATA, _BDATA, _CDATA))
        return data

    @property
    def rgb(self):
        """Read the RGB color detected by the sensor.  Returns a 3-tuple of
        red, green, blue component values as bytes (0-255).
        """
        r, g, b, clear = self.raw
        if clear:
            r = int(255 * r / clear)
            g = int(255 * g / clear)
            b = int(255 * b / clear)
        else:
            r, g, b = 0, 0, 0
        return r, g, b

    def _valid(self):
        """Check if RGBC is valid"""
        return self._read_byte(_STATUS) & 0x01

    def _read_byte(self, address):
        command = _CMD | address
        return self.bus.read_byte_data(self.address, command)

    def _read_word(self, address):
        command = _CMD | _AUTO | address
        return self.bus.read_word_data(self.address, command)

    def _write_byte(self, address, data):
        command = _CMD | address
        self.bus.write_byte_data(self.address, command, data)

    def _write_word(self, address, data):
        command = _CMD | _AUTO | address
        data = [(data >> 8) & 0xFF, data & 0xFF]
        self.bus.write_i2c_block_data(self.address, command, data)


Grove = GroveI2cColorSensorV2


def main():
    sensor = GroveI2cColorSensorV2()

    print('Raw data of red-filtered, green-filtered, blue-filtered and unfiltered photodiodes')
    while True:
        # r, g, b = sensor.rgb
        r, g, b, clear = sensor.raw
        print((r, g, b, clear))
        time.sleep(1.0)

if __name__ == '__main__':
    main()
